with GNATCOLL.Symbols;  use GNATCOLL;
with GNATCOLL.Utils;
with SPARK_Util;
with Sinput;            use Sinput;
with Why.Sinfo;         use Why.Sinfo;

package body Why.Atree.To_Json is

   ---------------------
   --  General types  --
   ---------------------

   procedure Boolean_To_Json         (O : Output_Id; B : Boolean);
   procedure Uint_To_Json            (O : Output_Id; I : Uint);
   procedure Ureal_To_Json           (O : Output_Id; I : Ureal);
   procedure String_Sets_Set_To_Json (O : Output_Id; S : String_Sets.Set);

   procedure Boolean_To_Json (O : Output_Id; B : Boolean) is
   begin
      if B then
         P (O, "true");
      else
         P (O, "false");
      end if;
   end Boolean_To_Json;

   procedure Uint_To_Json (O : Output_Id; I : Uint) is
   begin
      P (O, '"' & UI_Image (I, Decimal) & '"');
   end Uint_To_Json;

   procedure Ureal_To_Json (O : Output_Id; I : Ureal) is
   begin
      P (O, '[');
      Uint_To_Json (O, Numerator (I));
      P (O, ',');
      Uint_To_Json (O, Denominator (I));
      P (O, ',');
      P (O, Nat'Image (Rbase (I)));
      P (O, ',');
      Boolean_To_Json (O, UR_Is_Negative (I));
      P (O, ']');
   end Ureal_To_Json;

   procedure String_Sets_Set_To_Json (O : Output_Id; S : String_Sets.Set) is
      First : Boolean := True;
   begin
      P (O, '[');
         for El of S loop
            if First then
               First := False;
            else
               P (O, ',');
            end if;
            P (O, El, As_String => True);
         end loop;
      P (O, ']');
   end String_Sets_Set_To_Json;

   ------------------
   --  Gnat types  --
   ------------------

   procedure Source_Ptr_To_Json (O : Output_Id; X : Source_Ptr);
   procedure Symbol_To_Json (O : Output_Id; S : Symbols.Symbol);
   procedure Symbol_Set_To_Json (O : Output_Id; S : Symbol_Set);

   procedure Source_Ptr_To_Json (O : Output_Id; X : Source_Ptr) is
      File : constant String               := SPARK_Util.File_Name (X);
      Line : constant Physical_Line_Number := Get_Physical_Line_Number (X);
   begin
      if X = No_Location then
         P (O, "null");
         return;
      end if;
      P (O, '[');
      P (O, File, As_String => True);
      P (O, ',');
      P (O, Physical_Line_Number'Image (Line));
      P (O, ']');
   end Source_Ptr_To_Json;

   procedure Symbol_To_Json (O : Output_Id; S : Symbols.Symbol) is
      X : constant GNATCOLL.Utils.Cst_String_Access := Symbols.Get (S);
   begin
      P (O, X.all, As_String => True);
   end Symbol_To_Json;

   procedure Symbol_Set_To_Json (O : Output_Id; S : Symbol_Set) is
      First : Boolean := True;
   begin
      P (O, '[');
      for El of S loop
         if First then
            First := False;
         else
            P (O, ',');
         end if;
         Symbol_To_Json (O, El);
      end loop;
      P (O, ']');
   end Symbol_Set_To_Json;

   -----------------------------------------
   --  Why nodes types with multiplicity  --
   -----------------------------------------

   procedure Why_Node_Id_To_Json (O : Output_Id; Id : Why_Node_Id);
   procedure Why_Node_OId_To_Json (O : Output_Id; Id : Why_Node_Id);
   procedure Why_Node_OList_To_Json (O : Output_Id; Id : Why_Node_List);
   procedure Why_Node_List_To_Json (O : Output_Id; Id : Why_Node_List);

   procedure Why_Node_Id_To_Json (O : Output_Id; Id : Why_Node_Id) is
   begin
      Why_Node_To_Json (O, Node_Table (Id));
   end Why_Node_Id_To_Json;

   procedure Why_Node_OId_To_Json (O : Output_Id; Id : Why_Node_Id) is
   begin
      if Id = Why_Empty then
         P (O, "null");
      else
         Why_Node_Id_To_Json (O, Id);
      end if;
   end Why_Node_OId_To_Json;

   procedure Why_Node_OList_To_Json (O : Output_Id; Id : Why_Node_List) is
   begin
      if Is_Empty (Id) then
         P (O, "null");
      else
         Why_Node_List_To_Json (O, Id);
      end if;
   end Why_Node_OList_To_Json;

   procedure Why_Node_List_To_Json (O : Output_Id; Id : Why_Node_List) is
   begin
      Why_Node_Lists_List_To_Json (O, List_Table (Id).Content);
   end Why_Node_List_To_Json;

   procedure Why_Node_Lists_List_To_Json
     (O : Output_Id; L : Why_Node_Lists.List)
   is
      First : Boolean := True;
   begin
      P (O, '[');
      for El of L loop
         if First then
            First := False;
         else
            P (O, ',');
         end if;
         Why_Node_Id_To_Json (O, El);
      end loop;
      P (O, ']');
   end Why_Node_Lists_List_To_Json;

   pragma Warnings (Off, "procedure * is not referenced");
   --  Why.Sinfo

   procedure EW_Domain_To_Json
     (O : Output_Id;
      Arg : EW_Domain);

   procedure EW_Domain_To_Json
     (O : Output_Id;
      Arg : EW_Domain) is
   begin
      P (O, Integer'Image (EW_Domain'Enum_Rep (Arg)));
   end EW_Domain_To_Json;

   procedure EW_Type_To_Json
     (O : Output_Id;
      Arg : EW_Type);

   procedure EW_Type_To_Json
     (O : Output_Id;
      Arg : EW_Type) is
   begin
      P (O, Integer'Image (EW_Type'Enum_Rep (Arg)));
   end EW_Type_To_Json;

   procedure EW_Literal_To_Json
     (O : Output_Id;
      Arg : EW_Literal);

   procedure EW_Literal_To_Json
     (O : Output_Id;
      Arg : EW_Literal) is
   begin
      P (O, Integer'Image (EW_Literal'Enum_Rep (Arg)));
   end EW_Literal_To_Json;

   procedure EW_Theory_Type_To_Json
     (O : Output_Id;
      Arg : EW_Theory_Type);

   procedure EW_Theory_Type_To_Json
     (O : Output_Id;
      Arg : EW_Theory_Type) is
   begin
      P (O, Integer'Image (EW_Theory_Type'Enum_Rep (Arg)));
   end EW_Theory_Type_To_Json;

   procedure EW_Clone_Type_To_Json
     (O : Output_Id;
      Arg : EW_Clone_Type);

   procedure EW_Clone_Type_To_Json
     (O : Output_Id;
      Arg : EW_Clone_Type) is
   begin
      P (O, Integer'Image (EW_Clone_Type'Enum_Rep (Arg)));
   end EW_Clone_Type_To_Json;

   procedure EW_Subst_Type_To_Json
     (O : Output_Id;
      Arg : EW_Subst_Type);

   procedure EW_Subst_Type_To_Json
     (O : Output_Id;
      Arg : EW_Subst_Type) is
   begin
      P (O, Integer'Image (EW_Subst_Type'Enum_Rep (Arg)));
   end EW_Subst_Type_To_Json;

   procedure EW_Connector_To_Json
     (O : Output_Id;
      Arg : EW_Connector);

   procedure EW_Connector_To_Json
     (O : Output_Id;
      Arg : EW_Connector) is
   begin
      P (O, Integer'Image (EW_Connector'Enum_Rep (Arg)));
   end EW_Connector_To_Json;

   procedure EW_Assert_Kind_To_Json
     (O : Output_Id;
      Arg : EW_Assert_Kind);

   procedure EW_Assert_Kind_To_Json
     (O : Output_Id;
      Arg : EW_Assert_Kind) is
   begin
      P (O, Integer'Image (EW_Assert_Kind'Enum_Rep (Arg)));
   end EW_Assert_Kind_To_Json;

   procedure EW_Axiom_Dep_Kind_To_Json
     (O : Output_Id;
      Arg : EW_Axiom_Dep_Kind);

   procedure EW_Axiom_Dep_Kind_To_Json
     (O : Output_Id;
      Arg : EW_Axiom_Dep_Kind) is
   begin
      P (O, Integer'Image (EW_Axiom_Dep_Kind'Enum_Rep (Arg)));
   end EW_Axiom_Dep_Kind_To_Json;

   procedure W_Unused_At_Start_Opaque_OId_To_Json
     (O : Output_Id; Arg : W_Unused_At_Start_Opaque_OId);

   procedure W_Unused_At_Start_Opaque_OId_To_Json
     (O : Output_Id; Arg : W_Unused_At_Start_Opaque_OId)
   is
   begin
      Why_Node_OId_To_Json (O, Arg);
   end W_Unused_At_Start_Opaque_OId_To_Json;

   procedure W_Unused_At_Start_Opaque_OList_To_Json
     (O : Output_Id; Arg : W_Unused_At_Start_Opaque_OList);

   procedure W_Unused_At_Start_Opaque_OList_To_Json
     (O : Output_Id; Arg : W_Unused_At_Start_Opaque_OList)
   is
   begin
      Why_Node_OList_To_Json (O, Arg);
   end W_Unused_At_Start_Opaque_OList_To_Json;

   procedure W_Unused_At_Start_Opaque_Id_To_Json
     (O : Output_Id; Arg : W_Unused_At_Start_Opaque_Id);

   procedure W_Unused_At_Start_Opaque_Id_To_Json
     (O : Output_Id; Arg : W_Unused_At_Start_Opaque_Id)
   is
   begin
      Why_Node_Id_To_Json (O, Arg);
   end W_Unused_At_Start_Opaque_Id_To_Json;

   procedure W_Unused_At_Start_Opaque_List_To_Json
     (O : Output_Id; Arg : W_Unused_At_Start_Opaque_List);

   procedure W_Unused_At_Start_Opaque_List_To_Json
     (O : Output_Id; Arg : W_Unused_At_Start_Opaque_List)
   is
   begin
      Why_Node_List_To_Json (O, Arg);
   end W_Unused_At_Start_Opaque_List_To_Json;

   procedure W_Type_Opaque_OId_To_Json
     (O : Output_Id; Arg : W_Type_Opaque_OId);

   procedure W_Type_Opaque_OId_To_Json
     (O : Output_Id; Arg : W_Type_Opaque_OId)
   is
   begin
      Why_Node_OId_To_Json (O, Arg);
   end W_Type_Opaque_OId_To_Json;

   procedure W_Type_Opaque_OList_To_Json
     (O : Output_Id; Arg : W_Type_Opaque_OList);

   procedure W_Type_Opaque_OList_To_Json
     (O : Output_Id; Arg : W_Type_Opaque_OList)
   is
   begin
      Why_Node_OList_To_Json (O, Arg);
   end W_Type_Opaque_OList_To_Json;

   procedure W_Type_Opaque_Id_To_Json
     (O : Output_Id; Arg : W_Type_Opaque_Id);

   procedure W_Type_Opaque_Id_To_Json
     (O : Output_Id; Arg : W_Type_Opaque_Id)
   is
   begin
      Why_Node_Id_To_Json (O, Arg);
   end W_Type_Opaque_Id_To_Json;

   procedure W_Type_Opaque_List_To_Json
     (O : Output_Id; Arg : W_Type_Opaque_List);

   procedure W_Type_Opaque_List_To_Json
     (O : Output_Id; Arg : W_Type_Opaque_List)
   is
   begin
      Why_Node_List_To_Json (O, Arg);
   end W_Type_Opaque_List_To_Json;

   procedure W_Name_Opaque_OId_To_Json
     (O : Output_Id; Arg : W_Name_Opaque_OId);

   procedure W_Name_Opaque_OId_To_Json
     (O : Output_Id; Arg : W_Name_Opaque_OId)
   is
   begin
      Why_Node_OId_To_Json (O, Arg);
   end W_Name_Opaque_OId_To_Json;

   procedure W_Name_Opaque_OList_To_Json
     (O : Output_Id; Arg : W_Name_Opaque_OList);

   procedure W_Name_Opaque_OList_To_Json
     (O : Output_Id; Arg : W_Name_Opaque_OList)
   is
   begin
      Why_Node_OList_To_Json (O, Arg);
   end W_Name_Opaque_OList_To_Json;

   procedure W_Name_Opaque_Id_To_Json
     (O : Output_Id; Arg : W_Name_Opaque_Id);

   procedure W_Name_Opaque_Id_To_Json
     (O : Output_Id; Arg : W_Name_Opaque_Id)
   is
   begin
      Why_Node_Id_To_Json (O, Arg);
   end W_Name_Opaque_Id_To_Json;

   procedure W_Name_Opaque_List_To_Json
     (O : Output_Id; Arg : W_Name_Opaque_List);

   procedure W_Name_Opaque_List_To_Json
     (O : Output_Id; Arg : W_Name_Opaque_List)
   is
   begin
      Why_Node_List_To_Json (O, Arg);
   end W_Name_Opaque_List_To_Json;

   procedure W_Effects_Opaque_OId_To_Json
     (O : Output_Id; Arg : W_Effects_Opaque_OId);

   procedure W_Effects_Opaque_OId_To_Json
     (O : Output_Id; Arg : W_Effects_Opaque_OId)
   is
   begin
      Why_Node_OId_To_Json (O, Arg);
   end W_Effects_Opaque_OId_To_Json;

   procedure W_Effects_Opaque_OList_To_Json
     (O : Output_Id; Arg : W_Effects_Opaque_OList);

   procedure W_Effects_Opaque_OList_To_Json
     (O : Output_Id; Arg : W_Effects_Opaque_OList)
   is
   begin
      Why_Node_OList_To_Json (O, Arg);
   end W_Effects_Opaque_OList_To_Json;

   procedure W_Effects_Opaque_Id_To_Json
     (O : Output_Id; Arg : W_Effects_Opaque_Id);

   procedure W_Effects_Opaque_Id_To_Json
     (O : Output_Id; Arg : W_Effects_Opaque_Id)
   is
   begin
      Why_Node_Id_To_Json (O, Arg);
   end W_Effects_Opaque_Id_To_Json;

   procedure W_Effects_Opaque_List_To_Json
     (O : Output_Id; Arg : W_Effects_Opaque_List);

   procedure W_Effects_Opaque_List_To_Json
     (O : Output_Id; Arg : W_Effects_Opaque_List)
   is
   begin
      Why_Node_List_To_Json (O, Arg);
   end W_Effects_Opaque_List_To_Json;

   procedure W_Raise_Effect_Opaque_OId_To_Json
     (O : Output_Id; Arg : W_Raise_Effect_Opaque_OId);

   procedure W_Raise_Effect_Opaque_OId_To_Json
     (O : Output_Id; Arg : W_Raise_Effect_Opaque_OId)
   is
   begin
      Why_Node_OId_To_Json (O, Arg);
   end W_Raise_Effect_Opaque_OId_To_Json;

   procedure W_Raise_Effect_Opaque_OList_To_Json
     (O : Output_Id; Arg : W_Raise_Effect_Opaque_OList);

   procedure W_Raise_Effect_Opaque_OList_To_Json
     (O : Output_Id; Arg : W_Raise_Effect_Opaque_OList)
   is
   begin
      Why_Node_OList_To_Json (O, Arg);
   end W_Raise_Effect_Opaque_OList_To_Json;

   procedure W_Raise_Effect_Opaque_Id_To_Json
     (O : Output_Id; Arg : W_Raise_Effect_Opaque_Id);

   procedure W_Raise_Effect_Opaque_Id_To_Json
     (O : Output_Id; Arg : W_Raise_Effect_Opaque_Id)
   is
   begin
      Why_Node_Id_To_Json (O, Arg);
   end W_Raise_Effect_Opaque_Id_To_Json;

   procedure W_Raise_Effect_Opaque_List_To_Json
     (O : Output_Id; Arg : W_Raise_Effect_Opaque_List);

   procedure W_Raise_Effect_Opaque_List_To_Json
     (O : Output_Id; Arg : W_Raise_Effect_Opaque_List)
   is
   begin
      Why_Node_List_To_Json (O, Arg);
   end W_Raise_Effect_Opaque_List_To_Json;

   procedure W_Binder_Opaque_OId_To_Json
     (O : Output_Id; Arg : W_Binder_Opaque_OId);

   procedure W_Binder_Opaque_OId_To_Json
     (O : Output_Id; Arg : W_Binder_Opaque_OId)
   is
   begin
      Why_Node_OId_To_Json (O, Arg);
   end W_Binder_Opaque_OId_To_Json;

   procedure W_Binder_Opaque_OList_To_Json
     (O : Output_Id; Arg : W_Binder_Opaque_OList);

   procedure W_Binder_Opaque_OList_To_Json
     (O : Output_Id; Arg : W_Binder_Opaque_OList)
   is
   begin
      Why_Node_OList_To_Json (O, Arg);
   end W_Binder_Opaque_OList_To_Json;

   procedure W_Binder_Opaque_Id_To_Json
     (O : Output_Id; Arg : W_Binder_Opaque_Id);

   procedure W_Binder_Opaque_Id_To_Json
     (O : Output_Id; Arg : W_Binder_Opaque_Id)
   is
   begin
      Why_Node_Id_To_Json (O, Arg);
   end W_Binder_Opaque_Id_To_Json;

   procedure W_Binder_Opaque_List_To_Json
     (O : Output_Id; Arg : W_Binder_Opaque_List);

   procedure W_Binder_Opaque_List_To_Json
     (O : Output_Id; Arg : W_Binder_Opaque_List)
   is
   begin
      Why_Node_List_To_Json (O, Arg);
   end W_Binder_Opaque_List_To_Json;

   procedure W_Transparent_Type_Definition_Opaque_OId_To_Json
     (O : Output_Id; Arg : W_Transparent_Type_Definition_Opaque_OId);

   procedure W_Transparent_Type_Definition_Opaque_OId_To_Json
     (O : Output_Id; Arg : W_Transparent_Type_Definition_Opaque_OId)
   is
   begin
      Why_Node_OId_To_Json (O, Arg);
   end W_Transparent_Type_Definition_Opaque_OId_To_Json;

   procedure W_Transparent_Type_Definition_Opaque_OList_To_Json
     (O : Output_Id; Arg : W_Transparent_Type_Definition_Opaque_OList);

   procedure W_Transparent_Type_Definition_Opaque_OList_To_Json
     (O : Output_Id; Arg : W_Transparent_Type_Definition_Opaque_OList)
   is
   begin
      Why_Node_OList_To_Json (O, Arg);
   end W_Transparent_Type_Definition_Opaque_OList_To_Json;

   procedure W_Transparent_Type_Definition_Opaque_Id_To_Json
     (O : Output_Id; Arg : W_Transparent_Type_Definition_Opaque_Id);

   procedure W_Transparent_Type_Definition_Opaque_Id_To_Json
     (O : Output_Id; Arg : W_Transparent_Type_Definition_Opaque_Id)
   is
   begin
      Why_Node_Id_To_Json (O, Arg);
   end W_Transparent_Type_Definition_Opaque_Id_To_Json;

   procedure W_Transparent_Type_Definition_Opaque_List_To_Json
     (O : Output_Id; Arg : W_Transparent_Type_Definition_Opaque_List);

   procedure W_Transparent_Type_Definition_Opaque_List_To_Json
     (O : Output_Id; Arg : W_Transparent_Type_Definition_Opaque_List)
   is
   begin
      Why_Node_List_To_Json (O, Arg);
   end W_Transparent_Type_Definition_Opaque_List_To_Json;

   procedure W_Record_Binder_Opaque_OId_To_Json
     (O : Output_Id; Arg : W_Record_Binder_Opaque_OId);

   procedure W_Record_Binder_Opaque_OId_To_Json
     (O : Output_Id; Arg : W_Record_Binder_Opaque_OId)
   is
   begin
      Why_Node_OId_To_Json (O, Arg);
   end W_Record_Binder_Opaque_OId_To_Json;

   procedure W_Record_Binder_Opaque_OList_To_Json
     (O : Output_Id; Arg : W_Record_Binder_Opaque_OList);

   procedure W_Record_Binder_Opaque_OList_To_Json
     (O : Output_Id; Arg : W_Record_Binder_Opaque_OList)
   is
   begin
      Why_Node_OList_To_Json (O, Arg);
   end W_Record_Binder_Opaque_OList_To_Json;

   procedure W_Record_Binder_Opaque_Id_To_Json
     (O : Output_Id; Arg : W_Record_Binder_Opaque_Id);

   procedure W_Record_Binder_Opaque_Id_To_Json
     (O : Output_Id; Arg : W_Record_Binder_Opaque_Id)
   is
   begin
      Why_Node_Id_To_Json (O, Arg);
   end W_Record_Binder_Opaque_Id_To_Json;

   procedure W_Record_Binder_Opaque_List_To_Json
     (O : Output_Id; Arg : W_Record_Binder_Opaque_List);

   procedure W_Record_Binder_Opaque_List_To_Json
     (O : Output_Id; Arg : W_Record_Binder_Opaque_List)
   is
   begin
      Why_Node_List_To_Json (O, Arg);
   end W_Record_Binder_Opaque_List_To_Json;

   procedure W_Record_Definition_Opaque_OId_To_Json
     (O : Output_Id; Arg : W_Record_Definition_Opaque_OId);

   procedure W_Record_Definition_Opaque_OId_To_Json
     (O : Output_Id; Arg : W_Record_Definition_Opaque_OId)
   is
   begin
      Why_Node_OId_To_Json (O, Arg);
   end W_Record_Definition_Opaque_OId_To_Json;

   procedure W_Record_Definition_Opaque_OList_To_Json
     (O : Output_Id; Arg : W_Record_Definition_Opaque_OList);

   procedure W_Record_Definition_Opaque_OList_To_Json
     (O : Output_Id; Arg : W_Record_Definition_Opaque_OList)
   is
   begin
      Why_Node_OList_To_Json (O, Arg);
   end W_Record_Definition_Opaque_OList_To_Json;

   procedure W_Record_Definition_Opaque_Id_To_Json
     (O : Output_Id; Arg : W_Record_Definition_Opaque_Id);

   procedure W_Record_Definition_Opaque_Id_To_Json
     (O : Output_Id; Arg : W_Record_Definition_Opaque_Id)
   is
   begin
      Why_Node_Id_To_Json (O, Arg);
   end W_Record_Definition_Opaque_Id_To_Json;

   procedure W_Record_Definition_Opaque_List_To_Json
     (O : Output_Id; Arg : W_Record_Definition_Opaque_List);

   procedure W_Record_Definition_Opaque_List_To_Json
     (O : Output_Id; Arg : W_Record_Definition_Opaque_List)
   is
   begin
      Why_Node_List_To_Json (O, Arg);
   end W_Record_Definition_Opaque_List_To_Json;

   procedure W_Range_Type_Definition_Opaque_OId_To_Json
     (O : Output_Id; Arg : W_Range_Type_Definition_Opaque_OId);

   procedure W_Range_Type_Definition_Opaque_OId_To_Json
     (O : Output_Id; Arg : W_Range_Type_Definition_Opaque_OId)
   is
   begin
      Why_Node_OId_To_Json (O, Arg);
   end W_Range_Type_Definition_Opaque_OId_To_Json;

   procedure W_Range_Type_Definition_Opaque_OList_To_Json
     (O : Output_Id; Arg : W_Range_Type_Definition_Opaque_OList);

   procedure W_Range_Type_Definition_Opaque_OList_To_Json
     (O : Output_Id; Arg : W_Range_Type_Definition_Opaque_OList)
   is
   begin
      Why_Node_OList_To_Json (O, Arg);
   end W_Range_Type_Definition_Opaque_OList_To_Json;

   procedure W_Range_Type_Definition_Opaque_Id_To_Json
     (O : Output_Id; Arg : W_Range_Type_Definition_Opaque_Id);

   procedure W_Range_Type_Definition_Opaque_Id_To_Json
     (O : Output_Id; Arg : W_Range_Type_Definition_Opaque_Id)
   is
   begin
      Why_Node_Id_To_Json (O, Arg);
   end W_Range_Type_Definition_Opaque_Id_To_Json;

   procedure W_Range_Type_Definition_Opaque_List_To_Json
     (O : Output_Id; Arg : W_Range_Type_Definition_Opaque_List);

   procedure W_Range_Type_Definition_Opaque_List_To_Json
     (O : Output_Id; Arg : W_Range_Type_Definition_Opaque_List)
   is
   begin
      Why_Node_List_To_Json (O, Arg);
   end W_Range_Type_Definition_Opaque_List_To_Json;

   procedure W_Triggers_Opaque_OId_To_Json
     (O : Output_Id; Arg : W_Triggers_Opaque_OId);

   procedure W_Triggers_Opaque_OId_To_Json
     (O : Output_Id; Arg : W_Triggers_Opaque_OId)
   is
   begin
      Why_Node_OId_To_Json (O, Arg);
   end W_Triggers_Opaque_OId_To_Json;

   procedure W_Triggers_Opaque_OList_To_Json
     (O : Output_Id; Arg : W_Triggers_Opaque_OList);

   procedure W_Triggers_Opaque_OList_To_Json
     (O : Output_Id; Arg : W_Triggers_Opaque_OList)
   is
   begin
      Why_Node_OList_To_Json (O, Arg);
   end W_Triggers_Opaque_OList_To_Json;

   procedure W_Triggers_Opaque_Id_To_Json
     (O : Output_Id; Arg : W_Triggers_Opaque_Id);

   procedure W_Triggers_Opaque_Id_To_Json
     (O : Output_Id; Arg : W_Triggers_Opaque_Id)
   is
   begin
      Why_Node_Id_To_Json (O, Arg);
   end W_Triggers_Opaque_Id_To_Json;

   procedure W_Triggers_Opaque_List_To_Json
     (O : Output_Id; Arg : W_Triggers_Opaque_List);

   procedure W_Triggers_Opaque_List_To_Json
     (O : Output_Id; Arg : W_Triggers_Opaque_List)
   is
   begin
      Why_Node_List_To_Json (O, Arg);
   end W_Triggers_Opaque_List_To_Json;

   procedure W_Trigger_Opaque_OId_To_Json
     (O : Output_Id; Arg : W_Trigger_Opaque_OId);

   procedure W_Trigger_Opaque_OId_To_Json
     (O : Output_Id; Arg : W_Trigger_Opaque_OId)
   is
   begin
      Why_Node_OId_To_Json (O, Arg);
   end W_Trigger_Opaque_OId_To_Json;

   procedure W_Trigger_Opaque_OList_To_Json
     (O : Output_Id; Arg : W_Trigger_Opaque_OList);

   procedure W_Trigger_Opaque_OList_To_Json
     (O : Output_Id; Arg : W_Trigger_Opaque_OList)
   is
   begin
      Why_Node_OList_To_Json (O, Arg);
   end W_Trigger_Opaque_OList_To_Json;

   procedure W_Trigger_Opaque_Id_To_Json
     (O : Output_Id; Arg : W_Trigger_Opaque_Id);

   procedure W_Trigger_Opaque_Id_To_Json
     (O : Output_Id; Arg : W_Trigger_Opaque_Id)
   is
   begin
      Why_Node_Id_To_Json (O, Arg);
   end W_Trigger_Opaque_Id_To_Json;

   procedure W_Trigger_Opaque_List_To_Json
     (O : Output_Id; Arg : W_Trigger_Opaque_List);

   procedure W_Trigger_Opaque_List_To_Json
     (O : Output_Id; Arg : W_Trigger_Opaque_List)
   is
   begin
      Why_Node_List_To_Json (O, Arg);
   end W_Trigger_Opaque_List_To_Json;

   procedure W_Axiom_Dep_Opaque_OId_To_Json
     (O : Output_Id; Arg : W_Axiom_Dep_Opaque_OId);

   procedure W_Axiom_Dep_Opaque_OId_To_Json
     (O : Output_Id; Arg : W_Axiom_Dep_Opaque_OId)
   is
   begin
      Why_Node_OId_To_Json (O, Arg);
   end W_Axiom_Dep_Opaque_OId_To_Json;

   procedure W_Axiom_Dep_Opaque_OList_To_Json
     (O : Output_Id; Arg : W_Axiom_Dep_Opaque_OList);

   procedure W_Axiom_Dep_Opaque_OList_To_Json
     (O : Output_Id; Arg : W_Axiom_Dep_Opaque_OList)
   is
   begin
      Why_Node_OList_To_Json (O, Arg);
   end W_Axiom_Dep_Opaque_OList_To_Json;

   procedure W_Axiom_Dep_Opaque_Id_To_Json
     (O : Output_Id; Arg : W_Axiom_Dep_Opaque_Id);

   procedure W_Axiom_Dep_Opaque_Id_To_Json
     (O : Output_Id; Arg : W_Axiom_Dep_Opaque_Id)
   is
   begin
      Why_Node_Id_To_Json (O, Arg);
   end W_Axiom_Dep_Opaque_Id_To_Json;

   procedure W_Axiom_Dep_Opaque_List_To_Json
     (O : Output_Id; Arg : W_Axiom_Dep_Opaque_List);

   procedure W_Axiom_Dep_Opaque_List_To_Json
     (O : Output_Id; Arg : W_Axiom_Dep_Opaque_List)
   is
   begin
      Why_Node_List_To_Json (O, Arg);
   end W_Axiom_Dep_Opaque_List_To_Json;

   procedure W_Handler_Opaque_OId_To_Json
     (O : Output_Id; Arg : W_Handler_Opaque_OId);

   procedure W_Handler_Opaque_OId_To_Json
     (O : Output_Id; Arg : W_Handler_Opaque_OId)
   is
   begin
      Why_Node_OId_To_Json (O, Arg);
   end W_Handler_Opaque_OId_To_Json;

   procedure W_Handler_Opaque_OList_To_Json
     (O : Output_Id; Arg : W_Handler_Opaque_OList);

   procedure W_Handler_Opaque_OList_To_Json
     (O : Output_Id; Arg : W_Handler_Opaque_OList)
   is
   begin
      Why_Node_OList_To_Json (O, Arg);
   end W_Handler_Opaque_OList_To_Json;

   procedure W_Handler_Opaque_Id_To_Json
     (O : Output_Id; Arg : W_Handler_Opaque_Id);

   procedure W_Handler_Opaque_Id_To_Json
     (O : Output_Id; Arg : W_Handler_Opaque_Id)
   is
   begin
      Why_Node_Id_To_Json (O, Arg);
   end W_Handler_Opaque_Id_To_Json;

   procedure W_Handler_Opaque_List_To_Json
     (O : Output_Id; Arg : W_Handler_Opaque_List);

   procedure W_Handler_Opaque_List_To_Json
     (O : Output_Id; Arg : W_Handler_Opaque_List)
   is
   begin
      Why_Node_List_To_Json (O, Arg);
   end W_Handler_Opaque_List_To_Json;

   procedure W_Field_Association_Opaque_OId_To_Json
     (O : Output_Id; Arg : W_Field_Association_Opaque_OId);

   procedure W_Field_Association_Opaque_OId_To_Json
     (O : Output_Id; Arg : W_Field_Association_Opaque_OId)
   is
   begin
      Why_Node_OId_To_Json (O, Arg);
   end W_Field_Association_Opaque_OId_To_Json;

   procedure W_Field_Association_Opaque_OList_To_Json
     (O : Output_Id; Arg : W_Field_Association_Opaque_OList);

   procedure W_Field_Association_Opaque_OList_To_Json
     (O : Output_Id; Arg : W_Field_Association_Opaque_OList)
   is
   begin
      Why_Node_OList_To_Json (O, Arg);
   end W_Field_Association_Opaque_OList_To_Json;

   procedure W_Field_Association_Opaque_Id_To_Json
     (O : Output_Id; Arg : W_Field_Association_Opaque_Id);

   procedure W_Field_Association_Opaque_Id_To_Json
     (O : Output_Id; Arg : W_Field_Association_Opaque_Id)
   is
   begin
      Why_Node_Id_To_Json (O, Arg);
   end W_Field_Association_Opaque_Id_To_Json;

   procedure W_Field_Association_Opaque_List_To_Json
     (O : Output_Id; Arg : W_Field_Association_Opaque_List);

   procedure W_Field_Association_Opaque_List_To_Json
     (O : Output_Id; Arg : W_Field_Association_Opaque_List)
   is
   begin
      Why_Node_List_To_Json (O, Arg);
   end W_Field_Association_Opaque_List_To_Json;

   procedure W_Variant_Opaque_OId_To_Json
     (O : Output_Id; Arg : W_Variant_Opaque_OId);

   procedure W_Variant_Opaque_OId_To_Json
     (O : Output_Id; Arg : W_Variant_Opaque_OId)
   is
   begin
      Why_Node_OId_To_Json (O, Arg);
   end W_Variant_Opaque_OId_To_Json;

   procedure W_Variant_Opaque_OList_To_Json
     (O : Output_Id; Arg : W_Variant_Opaque_OList);

   procedure W_Variant_Opaque_OList_To_Json
     (O : Output_Id; Arg : W_Variant_Opaque_OList)
   is
   begin
      Why_Node_OList_To_Json (O, Arg);
   end W_Variant_Opaque_OList_To_Json;

   procedure W_Variant_Opaque_Id_To_Json
     (O : Output_Id; Arg : W_Variant_Opaque_Id);

   procedure W_Variant_Opaque_Id_To_Json
     (O : Output_Id; Arg : W_Variant_Opaque_Id)
   is
   begin
      Why_Node_Id_To_Json (O, Arg);
   end W_Variant_Opaque_Id_To_Json;

   procedure W_Variant_Opaque_List_To_Json
     (O : Output_Id; Arg : W_Variant_Opaque_List);

   procedure W_Variant_Opaque_List_To_Json
     (O : Output_Id; Arg : W_Variant_Opaque_List)
   is
   begin
      Why_Node_List_To_Json (O, Arg);
   end W_Variant_Opaque_List_To_Json;

   procedure W_Variants_Opaque_OId_To_Json
     (O : Output_Id; Arg : W_Variants_Opaque_OId);

   procedure W_Variants_Opaque_OId_To_Json
     (O : Output_Id; Arg : W_Variants_Opaque_OId)
   is
   begin
      Why_Node_OId_To_Json (O, Arg);
   end W_Variants_Opaque_OId_To_Json;

   procedure W_Variants_Opaque_OList_To_Json
     (O : Output_Id; Arg : W_Variants_Opaque_OList);

   procedure W_Variants_Opaque_OList_To_Json
     (O : Output_Id; Arg : W_Variants_Opaque_OList)
   is
   begin
      Why_Node_OList_To_Json (O, Arg);
   end W_Variants_Opaque_OList_To_Json;

   procedure W_Variants_Opaque_Id_To_Json
     (O : Output_Id; Arg : W_Variants_Opaque_Id);

   procedure W_Variants_Opaque_Id_To_Json
     (O : Output_Id; Arg : W_Variants_Opaque_Id)
   is
   begin
      Why_Node_Id_To_Json (O, Arg);
   end W_Variants_Opaque_Id_To_Json;

   procedure W_Variants_Opaque_List_To_Json
     (O : Output_Id; Arg : W_Variants_Opaque_List);

   procedure W_Variants_Opaque_List_To_Json
     (O : Output_Id; Arg : W_Variants_Opaque_List)
   is
   begin
      Why_Node_List_To_Json (O, Arg);
   end W_Variants_Opaque_List_To_Json;

   procedure W_Universal_Quantif_Opaque_OId_To_Json
     (O : Output_Id; Arg : W_Universal_Quantif_Opaque_OId);

   procedure W_Universal_Quantif_Opaque_OId_To_Json
     (O : Output_Id; Arg : W_Universal_Quantif_Opaque_OId)
   is
   begin
      Why_Node_OId_To_Json (O, Arg);
   end W_Universal_Quantif_Opaque_OId_To_Json;

   procedure W_Universal_Quantif_Opaque_OList_To_Json
     (O : Output_Id; Arg : W_Universal_Quantif_Opaque_OList);

   procedure W_Universal_Quantif_Opaque_OList_To_Json
     (O : Output_Id; Arg : W_Universal_Quantif_Opaque_OList)
   is
   begin
      Why_Node_OList_To_Json (O, Arg);
   end W_Universal_Quantif_Opaque_OList_To_Json;

   procedure W_Universal_Quantif_Opaque_Id_To_Json
     (O : Output_Id; Arg : W_Universal_Quantif_Opaque_Id);

   procedure W_Universal_Quantif_Opaque_Id_To_Json
     (O : Output_Id; Arg : W_Universal_Quantif_Opaque_Id)
   is
   begin
      Why_Node_Id_To_Json (O, Arg);
   end W_Universal_Quantif_Opaque_Id_To_Json;

   procedure W_Universal_Quantif_Opaque_List_To_Json
     (O : Output_Id; Arg : W_Universal_Quantif_Opaque_List);

   procedure W_Universal_Quantif_Opaque_List_To_Json
     (O : Output_Id; Arg : W_Universal_Quantif_Opaque_List)
   is
   begin
      Why_Node_List_To_Json (O, Arg);
   end W_Universal_Quantif_Opaque_List_To_Json;

   procedure W_Existential_Quantif_Opaque_OId_To_Json
     (O : Output_Id; Arg : W_Existential_Quantif_Opaque_OId);

   procedure W_Existential_Quantif_Opaque_OId_To_Json
     (O : Output_Id; Arg : W_Existential_Quantif_Opaque_OId)
   is
   begin
      Why_Node_OId_To_Json (O, Arg);
   end W_Existential_Quantif_Opaque_OId_To_Json;

   procedure W_Existential_Quantif_Opaque_OList_To_Json
     (O : Output_Id; Arg : W_Existential_Quantif_Opaque_OList);

   procedure W_Existential_Quantif_Opaque_OList_To_Json
     (O : Output_Id; Arg : W_Existential_Quantif_Opaque_OList)
   is
   begin
      Why_Node_OList_To_Json (O, Arg);
   end W_Existential_Quantif_Opaque_OList_To_Json;

   procedure W_Existential_Quantif_Opaque_Id_To_Json
     (O : Output_Id; Arg : W_Existential_Quantif_Opaque_Id);

   procedure W_Existential_Quantif_Opaque_Id_To_Json
     (O : Output_Id; Arg : W_Existential_Quantif_Opaque_Id)
   is
   begin
      Why_Node_Id_To_Json (O, Arg);
   end W_Existential_Quantif_Opaque_Id_To_Json;

   procedure W_Existential_Quantif_Opaque_List_To_Json
     (O : Output_Id; Arg : W_Existential_Quantif_Opaque_List);

   procedure W_Existential_Quantif_Opaque_List_To_Json
     (O : Output_Id; Arg : W_Existential_Quantif_Opaque_List)
   is
   begin
      Why_Node_List_To_Json (O, Arg);
   end W_Existential_Quantif_Opaque_List_To_Json;

   procedure W_Not_Opaque_OId_To_Json
     (O : Output_Id; Arg : W_Not_Opaque_OId);

   procedure W_Not_Opaque_OId_To_Json
     (O : Output_Id; Arg : W_Not_Opaque_OId)
   is
   begin
      Why_Node_OId_To_Json (O, Arg);
   end W_Not_Opaque_OId_To_Json;

   procedure W_Not_Opaque_OList_To_Json
     (O : Output_Id; Arg : W_Not_Opaque_OList);

   procedure W_Not_Opaque_OList_To_Json
     (O : Output_Id; Arg : W_Not_Opaque_OList)
   is
   begin
      Why_Node_OList_To_Json (O, Arg);
   end W_Not_Opaque_OList_To_Json;

   procedure W_Not_Opaque_Id_To_Json
     (O : Output_Id; Arg : W_Not_Opaque_Id);

   procedure W_Not_Opaque_Id_To_Json
     (O : Output_Id; Arg : W_Not_Opaque_Id)
   is
   begin
      Why_Node_Id_To_Json (O, Arg);
   end W_Not_Opaque_Id_To_Json;

   procedure W_Not_Opaque_List_To_Json
     (O : Output_Id; Arg : W_Not_Opaque_List);

   procedure W_Not_Opaque_List_To_Json
     (O : Output_Id; Arg : W_Not_Opaque_List)
   is
   begin
      Why_Node_List_To_Json (O, Arg);
   end W_Not_Opaque_List_To_Json;

   procedure W_Connection_Opaque_OId_To_Json
     (O : Output_Id; Arg : W_Connection_Opaque_OId);

   procedure W_Connection_Opaque_OId_To_Json
     (O : Output_Id; Arg : W_Connection_Opaque_OId)
   is
   begin
      Why_Node_OId_To_Json (O, Arg);
   end W_Connection_Opaque_OId_To_Json;

   procedure W_Connection_Opaque_OList_To_Json
     (O : Output_Id; Arg : W_Connection_Opaque_OList);

   procedure W_Connection_Opaque_OList_To_Json
     (O : Output_Id; Arg : W_Connection_Opaque_OList)
   is
   begin
      Why_Node_OList_To_Json (O, Arg);
   end W_Connection_Opaque_OList_To_Json;

   procedure W_Connection_Opaque_Id_To_Json
     (O : Output_Id; Arg : W_Connection_Opaque_Id);

   procedure W_Connection_Opaque_Id_To_Json
     (O : Output_Id; Arg : W_Connection_Opaque_Id)
   is
   begin
      Why_Node_Id_To_Json (O, Arg);
   end W_Connection_Opaque_Id_To_Json;

   procedure W_Connection_Opaque_List_To_Json
     (O : Output_Id; Arg : W_Connection_Opaque_List);

   procedure W_Connection_Opaque_List_To_Json
     (O : Output_Id; Arg : W_Connection_Opaque_List)
   is
   begin
      Why_Node_List_To_Json (O, Arg);
   end W_Connection_Opaque_List_To_Json;

   procedure W_Label_Opaque_OId_To_Json
     (O : Output_Id; Arg : W_Label_Opaque_OId);

   procedure W_Label_Opaque_OId_To_Json
     (O : Output_Id; Arg : W_Label_Opaque_OId)
   is
   begin
      Why_Node_OId_To_Json (O, Arg);
   end W_Label_Opaque_OId_To_Json;

   procedure W_Label_Opaque_OList_To_Json
     (O : Output_Id; Arg : W_Label_Opaque_OList);

   procedure W_Label_Opaque_OList_To_Json
     (O : Output_Id; Arg : W_Label_Opaque_OList)
   is
   begin
      Why_Node_OList_To_Json (O, Arg);
   end W_Label_Opaque_OList_To_Json;

   procedure W_Label_Opaque_Id_To_Json
     (O : Output_Id; Arg : W_Label_Opaque_Id);

   procedure W_Label_Opaque_Id_To_Json
     (O : Output_Id; Arg : W_Label_Opaque_Id)
   is
   begin
      Why_Node_Id_To_Json (O, Arg);
   end W_Label_Opaque_Id_To_Json;

   procedure W_Label_Opaque_List_To_Json
     (O : Output_Id; Arg : W_Label_Opaque_List);

   procedure W_Label_Opaque_List_To_Json
     (O : Output_Id; Arg : W_Label_Opaque_List)
   is
   begin
      Why_Node_List_To_Json (O, Arg);
   end W_Label_Opaque_List_To_Json;

   procedure W_Loc_Label_Opaque_OId_To_Json
     (O : Output_Id; Arg : W_Loc_Label_Opaque_OId);

   procedure W_Loc_Label_Opaque_OId_To_Json
     (O : Output_Id; Arg : W_Loc_Label_Opaque_OId)
   is
   begin
      Why_Node_OId_To_Json (O, Arg);
   end W_Loc_Label_Opaque_OId_To_Json;

   procedure W_Loc_Label_Opaque_OList_To_Json
     (O : Output_Id; Arg : W_Loc_Label_Opaque_OList);

   procedure W_Loc_Label_Opaque_OList_To_Json
     (O : Output_Id; Arg : W_Loc_Label_Opaque_OList)
   is
   begin
      Why_Node_OList_To_Json (O, Arg);
   end W_Loc_Label_Opaque_OList_To_Json;

   procedure W_Loc_Label_Opaque_Id_To_Json
     (O : Output_Id; Arg : W_Loc_Label_Opaque_Id);

   procedure W_Loc_Label_Opaque_Id_To_Json
     (O : Output_Id; Arg : W_Loc_Label_Opaque_Id)
   is
   begin
      Why_Node_Id_To_Json (O, Arg);
   end W_Loc_Label_Opaque_Id_To_Json;

   procedure W_Loc_Label_Opaque_List_To_Json
     (O : Output_Id; Arg : W_Loc_Label_Opaque_List);

   procedure W_Loc_Label_Opaque_List_To_Json
     (O : Output_Id; Arg : W_Loc_Label_Opaque_List)
   is
   begin
      Why_Node_List_To_Json (O, Arg);
   end W_Loc_Label_Opaque_List_To_Json;

   procedure W_Identifier_Opaque_OId_To_Json
     (O : Output_Id; Arg : W_Identifier_Opaque_OId);

   procedure W_Identifier_Opaque_OId_To_Json
     (O : Output_Id; Arg : W_Identifier_Opaque_OId)
   is
   begin
      Why_Node_OId_To_Json (O, Arg);
   end W_Identifier_Opaque_OId_To_Json;

   procedure W_Identifier_Opaque_OList_To_Json
     (O : Output_Id; Arg : W_Identifier_Opaque_OList);

   procedure W_Identifier_Opaque_OList_To_Json
     (O : Output_Id; Arg : W_Identifier_Opaque_OList)
   is
   begin
      Why_Node_OList_To_Json (O, Arg);
   end W_Identifier_Opaque_OList_To_Json;

   procedure W_Identifier_Opaque_Id_To_Json
     (O : Output_Id; Arg : W_Identifier_Opaque_Id);

   procedure W_Identifier_Opaque_Id_To_Json
     (O : Output_Id; Arg : W_Identifier_Opaque_Id)
   is
   begin
      Why_Node_Id_To_Json (O, Arg);
   end W_Identifier_Opaque_Id_To_Json;

   procedure W_Identifier_Opaque_List_To_Json
     (O : Output_Id; Arg : W_Identifier_Opaque_List);

   procedure W_Identifier_Opaque_List_To_Json
     (O : Output_Id; Arg : W_Identifier_Opaque_List)
   is
   begin
      Why_Node_List_To_Json (O, Arg);
   end W_Identifier_Opaque_List_To_Json;

   procedure W_Tagged_Opaque_OId_To_Json
     (O : Output_Id; Arg : W_Tagged_Opaque_OId);

   procedure W_Tagged_Opaque_OId_To_Json
     (O : Output_Id; Arg : W_Tagged_Opaque_OId)
   is
   begin
      Why_Node_OId_To_Json (O, Arg);
   end W_Tagged_Opaque_OId_To_Json;

   procedure W_Tagged_Opaque_OList_To_Json
     (O : Output_Id; Arg : W_Tagged_Opaque_OList);

   procedure W_Tagged_Opaque_OList_To_Json
     (O : Output_Id; Arg : W_Tagged_Opaque_OList)
   is
   begin
      Why_Node_OList_To_Json (O, Arg);
   end W_Tagged_Opaque_OList_To_Json;

   procedure W_Tagged_Opaque_Id_To_Json
     (O : Output_Id; Arg : W_Tagged_Opaque_Id);

   procedure W_Tagged_Opaque_Id_To_Json
     (O : Output_Id; Arg : W_Tagged_Opaque_Id)
   is
   begin
      Why_Node_Id_To_Json (O, Arg);
   end W_Tagged_Opaque_Id_To_Json;

   procedure W_Tagged_Opaque_List_To_Json
     (O : Output_Id; Arg : W_Tagged_Opaque_List);

   procedure W_Tagged_Opaque_List_To_Json
     (O : Output_Id; Arg : W_Tagged_Opaque_List)
   is
   begin
      Why_Node_List_To_Json (O, Arg);
   end W_Tagged_Opaque_List_To_Json;

   procedure W_Call_Opaque_OId_To_Json
     (O : Output_Id; Arg : W_Call_Opaque_OId);

   procedure W_Call_Opaque_OId_To_Json
     (O : Output_Id; Arg : W_Call_Opaque_OId)
   is
   begin
      Why_Node_OId_To_Json (O, Arg);
   end W_Call_Opaque_OId_To_Json;

   procedure W_Call_Opaque_OList_To_Json
     (O : Output_Id; Arg : W_Call_Opaque_OList);

   procedure W_Call_Opaque_OList_To_Json
     (O : Output_Id; Arg : W_Call_Opaque_OList)
   is
   begin
      Why_Node_OList_To_Json (O, Arg);
   end W_Call_Opaque_OList_To_Json;

   procedure W_Call_Opaque_Id_To_Json
     (O : Output_Id; Arg : W_Call_Opaque_Id);

   procedure W_Call_Opaque_Id_To_Json
     (O : Output_Id; Arg : W_Call_Opaque_Id)
   is
   begin
      Why_Node_Id_To_Json (O, Arg);
   end W_Call_Opaque_Id_To_Json;

   procedure W_Call_Opaque_List_To_Json
     (O : Output_Id; Arg : W_Call_Opaque_List);

   procedure W_Call_Opaque_List_To_Json
     (O : Output_Id; Arg : W_Call_Opaque_List)
   is
   begin
      Why_Node_List_To_Json (O, Arg);
   end W_Call_Opaque_List_To_Json;

   procedure W_Literal_Opaque_OId_To_Json
     (O : Output_Id; Arg : W_Literal_Opaque_OId);

   procedure W_Literal_Opaque_OId_To_Json
     (O : Output_Id; Arg : W_Literal_Opaque_OId)
   is
   begin
      Why_Node_OId_To_Json (O, Arg);
   end W_Literal_Opaque_OId_To_Json;

   procedure W_Literal_Opaque_OList_To_Json
     (O : Output_Id; Arg : W_Literal_Opaque_OList);

   procedure W_Literal_Opaque_OList_To_Json
     (O : Output_Id; Arg : W_Literal_Opaque_OList)
   is
   begin
      Why_Node_OList_To_Json (O, Arg);
   end W_Literal_Opaque_OList_To_Json;

   procedure W_Literal_Opaque_Id_To_Json
     (O : Output_Id; Arg : W_Literal_Opaque_Id);

   procedure W_Literal_Opaque_Id_To_Json
     (O : Output_Id; Arg : W_Literal_Opaque_Id)
   is
   begin
      Why_Node_Id_To_Json (O, Arg);
   end W_Literal_Opaque_Id_To_Json;

   procedure W_Literal_Opaque_List_To_Json
     (O : Output_Id; Arg : W_Literal_Opaque_List);

   procedure W_Literal_Opaque_List_To_Json
     (O : Output_Id; Arg : W_Literal_Opaque_List)
   is
   begin
      Why_Node_List_To_Json (O, Arg);
   end W_Literal_Opaque_List_To_Json;

   procedure W_Binding_Opaque_OId_To_Json
     (O : Output_Id; Arg : W_Binding_Opaque_OId);

   procedure W_Binding_Opaque_OId_To_Json
     (O : Output_Id; Arg : W_Binding_Opaque_OId)
   is
   begin
      Why_Node_OId_To_Json (O, Arg);
   end W_Binding_Opaque_OId_To_Json;

   procedure W_Binding_Opaque_OList_To_Json
     (O : Output_Id; Arg : W_Binding_Opaque_OList);

   procedure W_Binding_Opaque_OList_To_Json
     (O : Output_Id; Arg : W_Binding_Opaque_OList)
   is
   begin
      Why_Node_OList_To_Json (O, Arg);
   end W_Binding_Opaque_OList_To_Json;

   procedure W_Binding_Opaque_Id_To_Json
     (O : Output_Id; Arg : W_Binding_Opaque_Id);

   procedure W_Binding_Opaque_Id_To_Json
     (O : Output_Id; Arg : W_Binding_Opaque_Id)
   is
   begin
      Why_Node_Id_To_Json (O, Arg);
   end W_Binding_Opaque_Id_To_Json;

   procedure W_Binding_Opaque_List_To_Json
     (O : Output_Id; Arg : W_Binding_Opaque_List);

   procedure W_Binding_Opaque_List_To_Json
     (O : Output_Id; Arg : W_Binding_Opaque_List)
   is
   begin
      Why_Node_List_To_Json (O, Arg);
   end W_Binding_Opaque_List_To_Json;

   procedure W_Elsif_Opaque_OId_To_Json
     (O : Output_Id; Arg : W_Elsif_Opaque_OId);

   procedure W_Elsif_Opaque_OId_To_Json
     (O : Output_Id; Arg : W_Elsif_Opaque_OId)
   is
   begin
      Why_Node_OId_To_Json (O, Arg);
   end W_Elsif_Opaque_OId_To_Json;

   procedure W_Elsif_Opaque_OList_To_Json
     (O : Output_Id; Arg : W_Elsif_Opaque_OList);

   procedure W_Elsif_Opaque_OList_To_Json
     (O : Output_Id; Arg : W_Elsif_Opaque_OList)
   is
   begin
      Why_Node_OList_To_Json (O, Arg);
   end W_Elsif_Opaque_OList_To_Json;

   procedure W_Elsif_Opaque_Id_To_Json
     (O : Output_Id; Arg : W_Elsif_Opaque_Id);

   procedure W_Elsif_Opaque_Id_To_Json
     (O : Output_Id; Arg : W_Elsif_Opaque_Id)
   is
   begin
      Why_Node_Id_To_Json (O, Arg);
   end W_Elsif_Opaque_Id_To_Json;

   procedure W_Elsif_Opaque_List_To_Json
     (O : Output_Id; Arg : W_Elsif_Opaque_List);

   procedure W_Elsif_Opaque_List_To_Json
     (O : Output_Id; Arg : W_Elsif_Opaque_List)
   is
   begin
      Why_Node_List_To_Json (O, Arg);
   end W_Elsif_Opaque_List_To_Json;

   procedure W_Epsilon_Opaque_OId_To_Json
     (O : Output_Id; Arg : W_Epsilon_Opaque_OId);

   procedure W_Epsilon_Opaque_OId_To_Json
     (O : Output_Id; Arg : W_Epsilon_Opaque_OId)
   is
   begin
      Why_Node_OId_To_Json (O, Arg);
   end W_Epsilon_Opaque_OId_To_Json;

   procedure W_Epsilon_Opaque_OList_To_Json
     (O : Output_Id; Arg : W_Epsilon_Opaque_OList);

   procedure W_Epsilon_Opaque_OList_To_Json
     (O : Output_Id; Arg : W_Epsilon_Opaque_OList)
   is
   begin
      Why_Node_OList_To_Json (O, Arg);
   end W_Epsilon_Opaque_OList_To_Json;

   procedure W_Epsilon_Opaque_Id_To_Json
     (O : Output_Id; Arg : W_Epsilon_Opaque_Id);

   procedure W_Epsilon_Opaque_Id_To_Json
     (O : Output_Id; Arg : W_Epsilon_Opaque_Id)
   is
   begin
      Why_Node_Id_To_Json (O, Arg);
   end W_Epsilon_Opaque_Id_To_Json;

   procedure W_Epsilon_Opaque_List_To_Json
     (O : Output_Id; Arg : W_Epsilon_Opaque_List);

   procedure W_Epsilon_Opaque_List_To_Json
     (O : Output_Id; Arg : W_Epsilon_Opaque_List)
   is
   begin
      Why_Node_List_To_Json (O, Arg);
   end W_Epsilon_Opaque_List_To_Json;

   procedure W_Conditional_Opaque_OId_To_Json
     (O : Output_Id; Arg : W_Conditional_Opaque_OId);

   procedure W_Conditional_Opaque_OId_To_Json
     (O : Output_Id; Arg : W_Conditional_Opaque_OId)
   is
   begin
      Why_Node_OId_To_Json (O, Arg);
   end W_Conditional_Opaque_OId_To_Json;

   procedure W_Conditional_Opaque_OList_To_Json
     (O : Output_Id; Arg : W_Conditional_Opaque_OList);

   procedure W_Conditional_Opaque_OList_To_Json
     (O : Output_Id; Arg : W_Conditional_Opaque_OList)
   is
   begin
      Why_Node_OList_To_Json (O, Arg);
   end W_Conditional_Opaque_OList_To_Json;

   procedure W_Conditional_Opaque_Id_To_Json
     (O : Output_Id; Arg : W_Conditional_Opaque_Id);

   procedure W_Conditional_Opaque_Id_To_Json
     (O : Output_Id; Arg : W_Conditional_Opaque_Id)
   is
   begin
      Why_Node_Id_To_Json (O, Arg);
   end W_Conditional_Opaque_Id_To_Json;

   procedure W_Conditional_Opaque_List_To_Json
     (O : Output_Id; Arg : W_Conditional_Opaque_List);

   procedure W_Conditional_Opaque_List_To_Json
     (O : Output_Id; Arg : W_Conditional_Opaque_List)
   is
   begin
      Why_Node_List_To_Json (O, Arg);
   end W_Conditional_Opaque_List_To_Json;

   procedure W_Integer_Constant_Opaque_OId_To_Json
     (O : Output_Id; Arg : W_Integer_Constant_Opaque_OId);

   procedure W_Integer_Constant_Opaque_OId_To_Json
     (O : Output_Id; Arg : W_Integer_Constant_Opaque_OId)
   is
   begin
      Why_Node_OId_To_Json (O, Arg);
   end W_Integer_Constant_Opaque_OId_To_Json;

   procedure W_Integer_Constant_Opaque_OList_To_Json
     (O : Output_Id; Arg : W_Integer_Constant_Opaque_OList);

   procedure W_Integer_Constant_Opaque_OList_To_Json
     (O : Output_Id; Arg : W_Integer_Constant_Opaque_OList)
   is
   begin
      Why_Node_OList_To_Json (O, Arg);
   end W_Integer_Constant_Opaque_OList_To_Json;

   procedure W_Integer_Constant_Opaque_Id_To_Json
     (O : Output_Id; Arg : W_Integer_Constant_Opaque_Id);

   procedure W_Integer_Constant_Opaque_Id_To_Json
     (O : Output_Id; Arg : W_Integer_Constant_Opaque_Id)
   is
   begin
      Why_Node_Id_To_Json (O, Arg);
   end W_Integer_Constant_Opaque_Id_To_Json;

   procedure W_Integer_Constant_Opaque_List_To_Json
     (O : Output_Id; Arg : W_Integer_Constant_Opaque_List);

   procedure W_Integer_Constant_Opaque_List_To_Json
     (O : Output_Id; Arg : W_Integer_Constant_Opaque_List)
   is
   begin
      Why_Node_List_To_Json (O, Arg);
   end W_Integer_Constant_Opaque_List_To_Json;

   procedure W_Range_Constant_Opaque_OId_To_Json
     (O : Output_Id; Arg : W_Range_Constant_Opaque_OId);

   procedure W_Range_Constant_Opaque_OId_To_Json
     (O : Output_Id; Arg : W_Range_Constant_Opaque_OId)
   is
   begin
      Why_Node_OId_To_Json (O, Arg);
   end W_Range_Constant_Opaque_OId_To_Json;

   procedure W_Range_Constant_Opaque_OList_To_Json
     (O : Output_Id; Arg : W_Range_Constant_Opaque_OList);

   procedure W_Range_Constant_Opaque_OList_To_Json
     (O : Output_Id; Arg : W_Range_Constant_Opaque_OList)
   is
   begin
      Why_Node_OList_To_Json (O, Arg);
   end W_Range_Constant_Opaque_OList_To_Json;

   procedure W_Range_Constant_Opaque_Id_To_Json
     (O : Output_Id; Arg : W_Range_Constant_Opaque_Id);

   procedure W_Range_Constant_Opaque_Id_To_Json
     (O : Output_Id; Arg : W_Range_Constant_Opaque_Id)
   is
   begin
      Why_Node_Id_To_Json (O, Arg);
   end W_Range_Constant_Opaque_Id_To_Json;

   procedure W_Range_Constant_Opaque_List_To_Json
     (O : Output_Id; Arg : W_Range_Constant_Opaque_List);

   procedure W_Range_Constant_Opaque_List_To_Json
     (O : Output_Id; Arg : W_Range_Constant_Opaque_List)
   is
   begin
      Why_Node_List_To_Json (O, Arg);
   end W_Range_Constant_Opaque_List_To_Json;

   procedure W_Modular_Constant_Opaque_OId_To_Json
     (O : Output_Id; Arg : W_Modular_Constant_Opaque_OId);

   procedure W_Modular_Constant_Opaque_OId_To_Json
     (O : Output_Id; Arg : W_Modular_Constant_Opaque_OId)
   is
   begin
      Why_Node_OId_To_Json (O, Arg);
   end W_Modular_Constant_Opaque_OId_To_Json;

   procedure W_Modular_Constant_Opaque_OList_To_Json
     (O : Output_Id; Arg : W_Modular_Constant_Opaque_OList);

   procedure W_Modular_Constant_Opaque_OList_To_Json
     (O : Output_Id; Arg : W_Modular_Constant_Opaque_OList)
   is
   begin
      Why_Node_OList_To_Json (O, Arg);
   end W_Modular_Constant_Opaque_OList_To_Json;

   procedure W_Modular_Constant_Opaque_Id_To_Json
     (O : Output_Id; Arg : W_Modular_Constant_Opaque_Id);

   procedure W_Modular_Constant_Opaque_Id_To_Json
     (O : Output_Id; Arg : W_Modular_Constant_Opaque_Id)
   is
   begin
      Why_Node_Id_To_Json (O, Arg);
   end W_Modular_Constant_Opaque_Id_To_Json;

   procedure W_Modular_Constant_Opaque_List_To_Json
     (O : Output_Id; Arg : W_Modular_Constant_Opaque_List);

   procedure W_Modular_Constant_Opaque_List_To_Json
     (O : Output_Id; Arg : W_Modular_Constant_Opaque_List)
   is
   begin
      Why_Node_List_To_Json (O, Arg);
   end W_Modular_Constant_Opaque_List_To_Json;

   procedure W_Fixed_Constant_Opaque_OId_To_Json
     (O : Output_Id; Arg : W_Fixed_Constant_Opaque_OId);

   procedure W_Fixed_Constant_Opaque_OId_To_Json
     (O : Output_Id; Arg : W_Fixed_Constant_Opaque_OId)
   is
   begin
      Why_Node_OId_To_Json (O, Arg);
   end W_Fixed_Constant_Opaque_OId_To_Json;

   procedure W_Fixed_Constant_Opaque_OList_To_Json
     (O : Output_Id; Arg : W_Fixed_Constant_Opaque_OList);

   procedure W_Fixed_Constant_Opaque_OList_To_Json
     (O : Output_Id; Arg : W_Fixed_Constant_Opaque_OList)
   is
   begin
      Why_Node_OList_To_Json (O, Arg);
   end W_Fixed_Constant_Opaque_OList_To_Json;

   procedure W_Fixed_Constant_Opaque_Id_To_Json
     (O : Output_Id; Arg : W_Fixed_Constant_Opaque_Id);

   procedure W_Fixed_Constant_Opaque_Id_To_Json
     (O : Output_Id; Arg : W_Fixed_Constant_Opaque_Id)
   is
   begin
      Why_Node_Id_To_Json (O, Arg);
   end W_Fixed_Constant_Opaque_Id_To_Json;

   procedure W_Fixed_Constant_Opaque_List_To_Json
     (O : Output_Id; Arg : W_Fixed_Constant_Opaque_List);

   procedure W_Fixed_Constant_Opaque_List_To_Json
     (O : Output_Id; Arg : W_Fixed_Constant_Opaque_List)
   is
   begin
      Why_Node_List_To_Json (O, Arg);
   end W_Fixed_Constant_Opaque_List_To_Json;

   procedure W_Real_Constant_Opaque_OId_To_Json
     (O : Output_Id; Arg : W_Real_Constant_Opaque_OId);

   procedure W_Real_Constant_Opaque_OId_To_Json
     (O : Output_Id; Arg : W_Real_Constant_Opaque_OId)
   is
   begin
      Why_Node_OId_To_Json (O, Arg);
   end W_Real_Constant_Opaque_OId_To_Json;

   procedure W_Real_Constant_Opaque_OList_To_Json
     (O : Output_Id; Arg : W_Real_Constant_Opaque_OList);

   procedure W_Real_Constant_Opaque_OList_To_Json
     (O : Output_Id; Arg : W_Real_Constant_Opaque_OList)
   is
   begin
      Why_Node_OList_To_Json (O, Arg);
   end W_Real_Constant_Opaque_OList_To_Json;

   procedure W_Real_Constant_Opaque_Id_To_Json
     (O : Output_Id; Arg : W_Real_Constant_Opaque_Id);

   procedure W_Real_Constant_Opaque_Id_To_Json
     (O : Output_Id; Arg : W_Real_Constant_Opaque_Id)
   is
   begin
      Why_Node_Id_To_Json (O, Arg);
   end W_Real_Constant_Opaque_Id_To_Json;

   procedure W_Real_Constant_Opaque_List_To_Json
     (O : Output_Id; Arg : W_Real_Constant_Opaque_List);

   procedure W_Real_Constant_Opaque_List_To_Json
     (O : Output_Id; Arg : W_Real_Constant_Opaque_List)
   is
   begin
      Why_Node_List_To_Json (O, Arg);
   end W_Real_Constant_Opaque_List_To_Json;

   procedure W_Float_Constant_Opaque_OId_To_Json
     (O : Output_Id; Arg : W_Float_Constant_Opaque_OId);

   procedure W_Float_Constant_Opaque_OId_To_Json
     (O : Output_Id; Arg : W_Float_Constant_Opaque_OId)
   is
   begin
      Why_Node_OId_To_Json (O, Arg);
   end W_Float_Constant_Opaque_OId_To_Json;

   procedure W_Float_Constant_Opaque_OList_To_Json
     (O : Output_Id; Arg : W_Float_Constant_Opaque_OList);

   procedure W_Float_Constant_Opaque_OList_To_Json
     (O : Output_Id; Arg : W_Float_Constant_Opaque_OList)
   is
   begin
      Why_Node_OList_To_Json (O, Arg);
   end W_Float_Constant_Opaque_OList_To_Json;

   procedure W_Float_Constant_Opaque_Id_To_Json
     (O : Output_Id; Arg : W_Float_Constant_Opaque_Id);

   procedure W_Float_Constant_Opaque_Id_To_Json
     (O : Output_Id; Arg : W_Float_Constant_Opaque_Id)
   is
   begin
      Why_Node_Id_To_Json (O, Arg);
   end W_Float_Constant_Opaque_Id_To_Json;

   procedure W_Float_Constant_Opaque_List_To_Json
     (O : Output_Id; Arg : W_Float_Constant_Opaque_List);

   procedure W_Float_Constant_Opaque_List_To_Json
     (O : Output_Id; Arg : W_Float_Constant_Opaque_List)
   is
   begin
      Why_Node_List_To_Json (O, Arg);
   end W_Float_Constant_Opaque_List_To_Json;

   procedure W_Comment_Opaque_OId_To_Json
     (O : Output_Id; Arg : W_Comment_Opaque_OId);

   procedure W_Comment_Opaque_OId_To_Json
     (O : Output_Id; Arg : W_Comment_Opaque_OId)
   is
   begin
      Why_Node_OId_To_Json (O, Arg);
   end W_Comment_Opaque_OId_To_Json;

   procedure W_Comment_Opaque_OList_To_Json
     (O : Output_Id; Arg : W_Comment_Opaque_OList);

   procedure W_Comment_Opaque_OList_To_Json
     (O : Output_Id; Arg : W_Comment_Opaque_OList)
   is
   begin
      Why_Node_OList_To_Json (O, Arg);
   end W_Comment_Opaque_OList_To_Json;

   procedure W_Comment_Opaque_Id_To_Json
     (O : Output_Id; Arg : W_Comment_Opaque_Id);

   procedure W_Comment_Opaque_Id_To_Json
     (O : Output_Id; Arg : W_Comment_Opaque_Id)
   is
   begin
      Why_Node_Id_To_Json (O, Arg);
   end W_Comment_Opaque_Id_To_Json;

   procedure W_Comment_Opaque_List_To_Json
     (O : Output_Id; Arg : W_Comment_Opaque_List);

   procedure W_Comment_Opaque_List_To_Json
     (O : Output_Id; Arg : W_Comment_Opaque_List)
   is
   begin
      Why_Node_List_To_Json (O, Arg);
   end W_Comment_Opaque_List_To_Json;

   procedure W_Deref_Opaque_OId_To_Json
     (O : Output_Id; Arg : W_Deref_Opaque_OId);

   procedure W_Deref_Opaque_OId_To_Json
     (O : Output_Id; Arg : W_Deref_Opaque_OId)
   is
   begin
      Why_Node_OId_To_Json (O, Arg);
   end W_Deref_Opaque_OId_To_Json;

   procedure W_Deref_Opaque_OList_To_Json
     (O : Output_Id; Arg : W_Deref_Opaque_OList);

   procedure W_Deref_Opaque_OList_To_Json
     (O : Output_Id; Arg : W_Deref_Opaque_OList)
   is
   begin
      Why_Node_OList_To_Json (O, Arg);
   end W_Deref_Opaque_OList_To_Json;

   procedure W_Deref_Opaque_Id_To_Json
     (O : Output_Id; Arg : W_Deref_Opaque_Id);

   procedure W_Deref_Opaque_Id_To_Json
     (O : Output_Id; Arg : W_Deref_Opaque_Id)
   is
   begin
      Why_Node_Id_To_Json (O, Arg);
   end W_Deref_Opaque_Id_To_Json;

   procedure W_Deref_Opaque_List_To_Json
     (O : Output_Id; Arg : W_Deref_Opaque_List);

   procedure W_Deref_Opaque_List_To_Json
     (O : Output_Id; Arg : W_Deref_Opaque_List)
   is
   begin
      Why_Node_List_To_Json (O, Arg);
   end W_Deref_Opaque_List_To_Json;

   procedure W_Record_Access_Opaque_OId_To_Json
     (O : Output_Id; Arg : W_Record_Access_Opaque_OId);

   procedure W_Record_Access_Opaque_OId_To_Json
     (O : Output_Id; Arg : W_Record_Access_Opaque_OId)
   is
   begin
      Why_Node_OId_To_Json (O, Arg);
   end W_Record_Access_Opaque_OId_To_Json;

   procedure W_Record_Access_Opaque_OList_To_Json
     (O : Output_Id; Arg : W_Record_Access_Opaque_OList);

   procedure W_Record_Access_Opaque_OList_To_Json
     (O : Output_Id; Arg : W_Record_Access_Opaque_OList)
   is
   begin
      Why_Node_OList_To_Json (O, Arg);
   end W_Record_Access_Opaque_OList_To_Json;

   procedure W_Record_Access_Opaque_Id_To_Json
     (O : Output_Id; Arg : W_Record_Access_Opaque_Id);

   procedure W_Record_Access_Opaque_Id_To_Json
     (O : Output_Id; Arg : W_Record_Access_Opaque_Id)
   is
   begin
      Why_Node_Id_To_Json (O, Arg);
   end W_Record_Access_Opaque_Id_To_Json;

   procedure W_Record_Access_Opaque_List_To_Json
     (O : Output_Id; Arg : W_Record_Access_Opaque_List);

   procedure W_Record_Access_Opaque_List_To_Json
     (O : Output_Id; Arg : W_Record_Access_Opaque_List)
   is
   begin
      Why_Node_List_To_Json (O, Arg);
   end W_Record_Access_Opaque_List_To_Json;

   procedure W_Record_Update_Opaque_OId_To_Json
     (O : Output_Id; Arg : W_Record_Update_Opaque_OId);

   procedure W_Record_Update_Opaque_OId_To_Json
     (O : Output_Id; Arg : W_Record_Update_Opaque_OId)
   is
   begin
      Why_Node_OId_To_Json (O, Arg);
   end W_Record_Update_Opaque_OId_To_Json;

   procedure W_Record_Update_Opaque_OList_To_Json
     (O : Output_Id; Arg : W_Record_Update_Opaque_OList);

   procedure W_Record_Update_Opaque_OList_To_Json
     (O : Output_Id; Arg : W_Record_Update_Opaque_OList)
   is
   begin
      Why_Node_OList_To_Json (O, Arg);
   end W_Record_Update_Opaque_OList_To_Json;

   procedure W_Record_Update_Opaque_Id_To_Json
     (O : Output_Id; Arg : W_Record_Update_Opaque_Id);

   procedure W_Record_Update_Opaque_Id_To_Json
     (O : Output_Id; Arg : W_Record_Update_Opaque_Id)
   is
   begin
      Why_Node_Id_To_Json (O, Arg);
   end W_Record_Update_Opaque_Id_To_Json;

   procedure W_Record_Update_Opaque_List_To_Json
     (O : Output_Id; Arg : W_Record_Update_Opaque_List);

   procedure W_Record_Update_Opaque_List_To_Json
     (O : Output_Id; Arg : W_Record_Update_Opaque_List)
   is
   begin
      Why_Node_List_To_Json (O, Arg);
   end W_Record_Update_Opaque_List_To_Json;

   procedure W_Record_Aggregate_Opaque_OId_To_Json
     (O : Output_Id; Arg : W_Record_Aggregate_Opaque_OId);

   procedure W_Record_Aggregate_Opaque_OId_To_Json
     (O : Output_Id; Arg : W_Record_Aggregate_Opaque_OId)
   is
   begin
      Why_Node_OId_To_Json (O, Arg);
   end W_Record_Aggregate_Opaque_OId_To_Json;

   procedure W_Record_Aggregate_Opaque_OList_To_Json
     (O : Output_Id; Arg : W_Record_Aggregate_Opaque_OList);

   procedure W_Record_Aggregate_Opaque_OList_To_Json
     (O : Output_Id; Arg : W_Record_Aggregate_Opaque_OList)
   is
   begin
      Why_Node_OList_To_Json (O, Arg);
   end W_Record_Aggregate_Opaque_OList_To_Json;

   procedure W_Record_Aggregate_Opaque_Id_To_Json
     (O : Output_Id; Arg : W_Record_Aggregate_Opaque_Id);

   procedure W_Record_Aggregate_Opaque_Id_To_Json
     (O : Output_Id; Arg : W_Record_Aggregate_Opaque_Id)
   is
   begin
      Why_Node_Id_To_Json (O, Arg);
   end W_Record_Aggregate_Opaque_Id_To_Json;

   procedure W_Record_Aggregate_Opaque_List_To_Json
     (O : Output_Id; Arg : W_Record_Aggregate_Opaque_List);

   procedure W_Record_Aggregate_Opaque_List_To_Json
     (O : Output_Id; Arg : W_Record_Aggregate_Opaque_List)
   is
   begin
      Why_Node_List_To_Json (O, Arg);
   end W_Record_Aggregate_Opaque_List_To_Json;

   procedure W_Any_Expr_Opaque_OId_To_Json
     (O : Output_Id; Arg : W_Any_Expr_Opaque_OId);

   procedure W_Any_Expr_Opaque_OId_To_Json
     (O : Output_Id; Arg : W_Any_Expr_Opaque_OId)
   is
   begin
      Why_Node_OId_To_Json (O, Arg);
   end W_Any_Expr_Opaque_OId_To_Json;

   procedure W_Any_Expr_Opaque_OList_To_Json
     (O : Output_Id; Arg : W_Any_Expr_Opaque_OList);

   procedure W_Any_Expr_Opaque_OList_To_Json
     (O : Output_Id; Arg : W_Any_Expr_Opaque_OList)
   is
   begin
      Why_Node_OList_To_Json (O, Arg);
   end W_Any_Expr_Opaque_OList_To_Json;

   procedure W_Any_Expr_Opaque_Id_To_Json
     (O : Output_Id; Arg : W_Any_Expr_Opaque_Id);

   procedure W_Any_Expr_Opaque_Id_To_Json
     (O : Output_Id; Arg : W_Any_Expr_Opaque_Id)
   is
   begin
      Why_Node_Id_To_Json (O, Arg);
   end W_Any_Expr_Opaque_Id_To_Json;

   procedure W_Any_Expr_Opaque_List_To_Json
     (O : Output_Id; Arg : W_Any_Expr_Opaque_List);

   procedure W_Any_Expr_Opaque_List_To_Json
     (O : Output_Id; Arg : W_Any_Expr_Opaque_List)
   is
   begin
      Why_Node_List_To_Json (O, Arg);
   end W_Any_Expr_Opaque_List_To_Json;

   procedure W_Assignment_Opaque_OId_To_Json
     (O : Output_Id; Arg : W_Assignment_Opaque_OId);

   procedure W_Assignment_Opaque_OId_To_Json
     (O : Output_Id; Arg : W_Assignment_Opaque_OId)
   is
   begin
      Why_Node_OId_To_Json (O, Arg);
   end W_Assignment_Opaque_OId_To_Json;

   procedure W_Assignment_Opaque_OList_To_Json
     (O : Output_Id; Arg : W_Assignment_Opaque_OList);

   procedure W_Assignment_Opaque_OList_To_Json
     (O : Output_Id; Arg : W_Assignment_Opaque_OList)
   is
   begin
      Why_Node_OList_To_Json (O, Arg);
   end W_Assignment_Opaque_OList_To_Json;

   procedure W_Assignment_Opaque_Id_To_Json
     (O : Output_Id; Arg : W_Assignment_Opaque_Id);

   procedure W_Assignment_Opaque_Id_To_Json
     (O : Output_Id; Arg : W_Assignment_Opaque_Id)
   is
   begin
      Why_Node_Id_To_Json (O, Arg);
   end W_Assignment_Opaque_Id_To_Json;

   procedure W_Assignment_Opaque_List_To_Json
     (O : Output_Id; Arg : W_Assignment_Opaque_List);

   procedure W_Assignment_Opaque_List_To_Json
     (O : Output_Id; Arg : W_Assignment_Opaque_List)
   is
   begin
      Why_Node_List_To_Json (O, Arg);
   end W_Assignment_Opaque_List_To_Json;

   procedure W_Binding_Ref_Opaque_OId_To_Json
     (O : Output_Id; Arg : W_Binding_Ref_Opaque_OId);

   procedure W_Binding_Ref_Opaque_OId_To_Json
     (O : Output_Id; Arg : W_Binding_Ref_Opaque_OId)
   is
   begin
      Why_Node_OId_To_Json (O, Arg);
   end W_Binding_Ref_Opaque_OId_To_Json;

   procedure W_Binding_Ref_Opaque_OList_To_Json
     (O : Output_Id; Arg : W_Binding_Ref_Opaque_OList);

   procedure W_Binding_Ref_Opaque_OList_To_Json
     (O : Output_Id; Arg : W_Binding_Ref_Opaque_OList)
   is
   begin
      Why_Node_OList_To_Json (O, Arg);
   end W_Binding_Ref_Opaque_OList_To_Json;

   procedure W_Binding_Ref_Opaque_Id_To_Json
     (O : Output_Id; Arg : W_Binding_Ref_Opaque_Id);

   procedure W_Binding_Ref_Opaque_Id_To_Json
     (O : Output_Id; Arg : W_Binding_Ref_Opaque_Id)
   is
   begin
      Why_Node_Id_To_Json (O, Arg);
   end W_Binding_Ref_Opaque_Id_To_Json;

   procedure W_Binding_Ref_Opaque_List_To_Json
     (O : Output_Id; Arg : W_Binding_Ref_Opaque_List);

   procedure W_Binding_Ref_Opaque_List_To_Json
     (O : Output_Id; Arg : W_Binding_Ref_Opaque_List)
   is
   begin
      Why_Node_List_To_Json (O, Arg);
   end W_Binding_Ref_Opaque_List_To_Json;

   procedure W_Loop_Opaque_OId_To_Json
     (O : Output_Id; Arg : W_Loop_Opaque_OId);

   procedure W_Loop_Opaque_OId_To_Json
     (O : Output_Id; Arg : W_Loop_Opaque_OId)
   is
   begin
      Why_Node_OId_To_Json (O, Arg);
   end W_Loop_Opaque_OId_To_Json;

   procedure W_Loop_Opaque_OList_To_Json
     (O : Output_Id; Arg : W_Loop_Opaque_OList);

   procedure W_Loop_Opaque_OList_To_Json
     (O : Output_Id; Arg : W_Loop_Opaque_OList)
   is
   begin
      Why_Node_OList_To_Json (O, Arg);
   end W_Loop_Opaque_OList_To_Json;

   procedure W_Loop_Opaque_Id_To_Json
     (O : Output_Id; Arg : W_Loop_Opaque_Id);

   procedure W_Loop_Opaque_Id_To_Json
     (O : Output_Id; Arg : W_Loop_Opaque_Id)
   is
   begin
      Why_Node_Id_To_Json (O, Arg);
   end W_Loop_Opaque_Id_To_Json;

   procedure W_Loop_Opaque_List_To_Json
     (O : Output_Id; Arg : W_Loop_Opaque_List);

   procedure W_Loop_Opaque_List_To_Json
     (O : Output_Id; Arg : W_Loop_Opaque_List)
   is
   begin
      Why_Node_List_To_Json (O, Arg);
   end W_Loop_Opaque_List_To_Json;

   procedure W_Statement_Sequence_Opaque_OId_To_Json
     (O : Output_Id; Arg : W_Statement_Sequence_Opaque_OId);

   procedure W_Statement_Sequence_Opaque_OId_To_Json
     (O : Output_Id; Arg : W_Statement_Sequence_Opaque_OId)
   is
   begin
      Why_Node_OId_To_Json (O, Arg);
   end W_Statement_Sequence_Opaque_OId_To_Json;

   procedure W_Statement_Sequence_Opaque_OList_To_Json
     (O : Output_Id; Arg : W_Statement_Sequence_Opaque_OList);

   procedure W_Statement_Sequence_Opaque_OList_To_Json
     (O : Output_Id; Arg : W_Statement_Sequence_Opaque_OList)
   is
   begin
      Why_Node_OList_To_Json (O, Arg);
   end W_Statement_Sequence_Opaque_OList_To_Json;

   procedure W_Statement_Sequence_Opaque_Id_To_Json
     (O : Output_Id; Arg : W_Statement_Sequence_Opaque_Id);

   procedure W_Statement_Sequence_Opaque_Id_To_Json
     (O : Output_Id; Arg : W_Statement_Sequence_Opaque_Id)
   is
   begin
      Why_Node_Id_To_Json (O, Arg);
   end W_Statement_Sequence_Opaque_Id_To_Json;

   procedure W_Statement_Sequence_Opaque_List_To_Json
     (O : Output_Id; Arg : W_Statement_Sequence_Opaque_List);

   procedure W_Statement_Sequence_Opaque_List_To_Json
     (O : Output_Id; Arg : W_Statement_Sequence_Opaque_List)
   is
   begin
      Why_Node_List_To_Json (O, Arg);
   end W_Statement_Sequence_Opaque_List_To_Json;

   procedure W_Abstract_Expr_Opaque_OId_To_Json
     (O : Output_Id; Arg : W_Abstract_Expr_Opaque_OId);

   procedure W_Abstract_Expr_Opaque_OId_To_Json
     (O : Output_Id; Arg : W_Abstract_Expr_Opaque_OId)
   is
   begin
      Why_Node_OId_To_Json (O, Arg);
   end W_Abstract_Expr_Opaque_OId_To_Json;

   procedure W_Abstract_Expr_Opaque_OList_To_Json
     (O : Output_Id; Arg : W_Abstract_Expr_Opaque_OList);

   procedure W_Abstract_Expr_Opaque_OList_To_Json
     (O : Output_Id; Arg : W_Abstract_Expr_Opaque_OList)
   is
   begin
      Why_Node_OList_To_Json (O, Arg);
   end W_Abstract_Expr_Opaque_OList_To_Json;

   procedure W_Abstract_Expr_Opaque_Id_To_Json
     (O : Output_Id; Arg : W_Abstract_Expr_Opaque_Id);

   procedure W_Abstract_Expr_Opaque_Id_To_Json
     (O : Output_Id; Arg : W_Abstract_Expr_Opaque_Id)
   is
   begin
      Why_Node_Id_To_Json (O, Arg);
   end W_Abstract_Expr_Opaque_Id_To_Json;

   procedure W_Abstract_Expr_Opaque_List_To_Json
     (O : Output_Id; Arg : W_Abstract_Expr_Opaque_List);

   procedure W_Abstract_Expr_Opaque_List_To_Json
     (O : Output_Id; Arg : W_Abstract_Expr_Opaque_List)
   is
   begin
      Why_Node_List_To_Json (O, Arg);
   end W_Abstract_Expr_Opaque_List_To_Json;

   procedure W_Assert_Opaque_OId_To_Json
     (O : Output_Id; Arg : W_Assert_Opaque_OId);

   procedure W_Assert_Opaque_OId_To_Json
     (O : Output_Id; Arg : W_Assert_Opaque_OId)
   is
   begin
      Why_Node_OId_To_Json (O, Arg);
   end W_Assert_Opaque_OId_To_Json;

   procedure W_Assert_Opaque_OList_To_Json
     (O : Output_Id; Arg : W_Assert_Opaque_OList);

   procedure W_Assert_Opaque_OList_To_Json
     (O : Output_Id; Arg : W_Assert_Opaque_OList)
   is
   begin
      Why_Node_OList_To_Json (O, Arg);
   end W_Assert_Opaque_OList_To_Json;

   procedure W_Assert_Opaque_Id_To_Json
     (O : Output_Id; Arg : W_Assert_Opaque_Id);

   procedure W_Assert_Opaque_Id_To_Json
     (O : Output_Id; Arg : W_Assert_Opaque_Id)
   is
   begin
      Why_Node_Id_To_Json (O, Arg);
   end W_Assert_Opaque_Id_To_Json;

   procedure W_Assert_Opaque_List_To_Json
     (O : Output_Id; Arg : W_Assert_Opaque_List);

   procedure W_Assert_Opaque_List_To_Json
     (O : Output_Id; Arg : W_Assert_Opaque_List)
   is
   begin
      Why_Node_List_To_Json (O, Arg);
   end W_Assert_Opaque_List_To_Json;

   procedure W_Raise_Opaque_OId_To_Json
     (O : Output_Id; Arg : W_Raise_Opaque_OId);

   procedure W_Raise_Opaque_OId_To_Json
     (O : Output_Id; Arg : W_Raise_Opaque_OId)
   is
   begin
      Why_Node_OId_To_Json (O, Arg);
   end W_Raise_Opaque_OId_To_Json;

   procedure W_Raise_Opaque_OList_To_Json
     (O : Output_Id; Arg : W_Raise_Opaque_OList);

   procedure W_Raise_Opaque_OList_To_Json
     (O : Output_Id; Arg : W_Raise_Opaque_OList)
   is
   begin
      Why_Node_OList_To_Json (O, Arg);
   end W_Raise_Opaque_OList_To_Json;

   procedure W_Raise_Opaque_Id_To_Json
     (O : Output_Id; Arg : W_Raise_Opaque_Id);

   procedure W_Raise_Opaque_Id_To_Json
     (O : Output_Id; Arg : W_Raise_Opaque_Id)
   is
   begin
      Why_Node_Id_To_Json (O, Arg);
   end W_Raise_Opaque_Id_To_Json;

   procedure W_Raise_Opaque_List_To_Json
     (O : Output_Id; Arg : W_Raise_Opaque_List);

   procedure W_Raise_Opaque_List_To_Json
     (O : Output_Id; Arg : W_Raise_Opaque_List)
   is
   begin
      Why_Node_List_To_Json (O, Arg);
   end W_Raise_Opaque_List_To_Json;

   procedure W_Try_Block_Opaque_OId_To_Json
     (O : Output_Id; Arg : W_Try_Block_Opaque_OId);

   procedure W_Try_Block_Opaque_OId_To_Json
     (O : Output_Id; Arg : W_Try_Block_Opaque_OId)
   is
   begin
      Why_Node_OId_To_Json (O, Arg);
   end W_Try_Block_Opaque_OId_To_Json;

   procedure W_Try_Block_Opaque_OList_To_Json
     (O : Output_Id; Arg : W_Try_Block_Opaque_OList);

   procedure W_Try_Block_Opaque_OList_To_Json
     (O : Output_Id; Arg : W_Try_Block_Opaque_OList)
   is
   begin
      Why_Node_OList_To_Json (O, Arg);
   end W_Try_Block_Opaque_OList_To_Json;

   procedure W_Try_Block_Opaque_Id_To_Json
     (O : Output_Id; Arg : W_Try_Block_Opaque_Id);

   procedure W_Try_Block_Opaque_Id_To_Json
     (O : Output_Id; Arg : W_Try_Block_Opaque_Id)
   is
   begin
      Why_Node_Id_To_Json (O, Arg);
   end W_Try_Block_Opaque_Id_To_Json;

   procedure W_Try_Block_Opaque_List_To_Json
     (O : Output_Id; Arg : W_Try_Block_Opaque_List);

   procedure W_Try_Block_Opaque_List_To_Json
     (O : Output_Id; Arg : W_Try_Block_Opaque_List)
   is
   begin
      Why_Node_List_To_Json (O, Arg);
   end W_Try_Block_Opaque_List_To_Json;

   procedure W_Function_Decl_Opaque_OId_To_Json
     (O : Output_Id; Arg : W_Function_Decl_Opaque_OId);

   procedure W_Function_Decl_Opaque_OId_To_Json
     (O : Output_Id; Arg : W_Function_Decl_Opaque_OId)
   is
   begin
      Why_Node_OId_To_Json (O, Arg);
   end W_Function_Decl_Opaque_OId_To_Json;

   procedure W_Function_Decl_Opaque_OList_To_Json
     (O : Output_Id; Arg : W_Function_Decl_Opaque_OList);

   procedure W_Function_Decl_Opaque_OList_To_Json
     (O : Output_Id; Arg : W_Function_Decl_Opaque_OList)
   is
   begin
      Why_Node_OList_To_Json (O, Arg);
   end W_Function_Decl_Opaque_OList_To_Json;

   procedure W_Function_Decl_Opaque_Id_To_Json
     (O : Output_Id; Arg : W_Function_Decl_Opaque_Id);

   procedure W_Function_Decl_Opaque_Id_To_Json
     (O : Output_Id; Arg : W_Function_Decl_Opaque_Id)
   is
   begin
      Why_Node_Id_To_Json (O, Arg);
   end W_Function_Decl_Opaque_Id_To_Json;

   procedure W_Function_Decl_Opaque_List_To_Json
     (O : Output_Id; Arg : W_Function_Decl_Opaque_List);

   procedure W_Function_Decl_Opaque_List_To_Json
     (O : Output_Id; Arg : W_Function_Decl_Opaque_List)
   is
   begin
      Why_Node_List_To_Json (O, Arg);
   end W_Function_Decl_Opaque_List_To_Json;

   procedure W_Axiom_Opaque_OId_To_Json
     (O : Output_Id; Arg : W_Axiom_Opaque_OId);

   procedure W_Axiom_Opaque_OId_To_Json
     (O : Output_Id; Arg : W_Axiom_Opaque_OId)
   is
   begin
      Why_Node_OId_To_Json (O, Arg);
   end W_Axiom_Opaque_OId_To_Json;

   procedure W_Axiom_Opaque_OList_To_Json
     (O : Output_Id; Arg : W_Axiom_Opaque_OList);

   procedure W_Axiom_Opaque_OList_To_Json
     (O : Output_Id; Arg : W_Axiom_Opaque_OList)
   is
   begin
      Why_Node_OList_To_Json (O, Arg);
   end W_Axiom_Opaque_OList_To_Json;

   procedure W_Axiom_Opaque_Id_To_Json
     (O : Output_Id; Arg : W_Axiom_Opaque_Id);

   procedure W_Axiom_Opaque_Id_To_Json
     (O : Output_Id; Arg : W_Axiom_Opaque_Id)
   is
   begin
      Why_Node_Id_To_Json (O, Arg);
   end W_Axiom_Opaque_Id_To_Json;

   procedure W_Axiom_Opaque_List_To_Json
     (O : Output_Id; Arg : W_Axiom_Opaque_List);

   procedure W_Axiom_Opaque_List_To_Json
     (O : Output_Id; Arg : W_Axiom_Opaque_List)
   is
   begin
      Why_Node_List_To_Json (O, Arg);
   end W_Axiom_Opaque_List_To_Json;

   procedure W_Goal_Opaque_OId_To_Json
     (O : Output_Id; Arg : W_Goal_Opaque_OId);

   procedure W_Goal_Opaque_OId_To_Json
     (O : Output_Id; Arg : W_Goal_Opaque_OId)
   is
   begin
      Why_Node_OId_To_Json (O, Arg);
   end W_Goal_Opaque_OId_To_Json;

   procedure W_Goal_Opaque_OList_To_Json
     (O : Output_Id; Arg : W_Goal_Opaque_OList);

   procedure W_Goal_Opaque_OList_To_Json
     (O : Output_Id; Arg : W_Goal_Opaque_OList)
   is
   begin
      Why_Node_OList_To_Json (O, Arg);
   end W_Goal_Opaque_OList_To_Json;

   procedure W_Goal_Opaque_Id_To_Json
     (O : Output_Id; Arg : W_Goal_Opaque_Id);

   procedure W_Goal_Opaque_Id_To_Json
     (O : Output_Id; Arg : W_Goal_Opaque_Id)
   is
   begin
      Why_Node_Id_To_Json (O, Arg);
   end W_Goal_Opaque_Id_To_Json;

   procedure W_Goal_Opaque_List_To_Json
     (O : Output_Id; Arg : W_Goal_Opaque_List);

   procedure W_Goal_Opaque_List_To_Json
     (O : Output_Id; Arg : W_Goal_Opaque_List)
   is
   begin
      Why_Node_List_To_Json (O, Arg);
   end W_Goal_Opaque_List_To_Json;

   procedure W_Type_Decl_Opaque_OId_To_Json
     (O : Output_Id; Arg : W_Type_Decl_Opaque_OId);

   procedure W_Type_Decl_Opaque_OId_To_Json
     (O : Output_Id; Arg : W_Type_Decl_Opaque_OId)
   is
   begin
      Why_Node_OId_To_Json (O, Arg);
   end W_Type_Decl_Opaque_OId_To_Json;

   procedure W_Type_Decl_Opaque_OList_To_Json
     (O : Output_Id; Arg : W_Type_Decl_Opaque_OList);

   procedure W_Type_Decl_Opaque_OList_To_Json
     (O : Output_Id; Arg : W_Type_Decl_Opaque_OList)
   is
   begin
      Why_Node_OList_To_Json (O, Arg);
   end W_Type_Decl_Opaque_OList_To_Json;

   procedure W_Type_Decl_Opaque_Id_To_Json
     (O : Output_Id; Arg : W_Type_Decl_Opaque_Id);

   procedure W_Type_Decl_Opaque_Id_To_Json
     (O : Output_Id; Arg : W_Type_Decl_Opaque_Id)
   is
   begin
      Why_Node_Id_To_Json (O, Arg);
   end W_Type_Decl_Opaque_Id_To_Json;

   procedure W_Type_Decl_Opaque_List_To_Json
     (O : Output_Id; Arg : W_Type_Decl_Opaque_List);

   procedure W_Type_Decl_Opaque_List_To_Json
     (O : Output_Id; Arg : W_Type_Decl_Opaque_List)
   is
   begin
      Why_Node_List_To_Json (O, Arg);
   end W_Type_Decl_Opaque_List_To_Json;

   procedure W_Global_Ref_Declaration_Opaque_OId_To_Json
     (O : Output_Id; Arg : W_Global_Ref_Declaration_Opaque_OId);

   procedure W_Global_Ref_Declaration_Opaque_OId_To_Json
     (O : Output_Id; Arg : W_Global_Ref_Declaration_Opaque_OId)
   is
   begin
      Why_Node_OId_To_Json (O, Arg);
   end W_Global_Ref_Declaration_Opaque_OId_To_Json;

   procedure W_Global_Ref_Declaration_Opaque_OList_To_Json
     (O : Output_Id; Arg : W_Global_Ref_Declaration_Opaque_OList);

   procedure W_Global_Ref_Declaration_Opaque_OList_To_Json
     (O : Output_Id; Arg : W_Global_Ref_Declaration_Opaque_OList)
   is
   begin
      Why_Node_OList_To_Json (O, Arg);
   end W_Global_Ref_Declaration_Opaque_OList_To_Json;

   procedure W_Global_Ref_Declaration_Opaque_Id_To_Json
     (O : Output_Id; Arg : W_Global_Ref_Declaration_Opaque_Id);

   procedure W_Global_Ref_Declaration_Opaque_Id_To_Json
     (O : Output_Id; Arg : W_Global_Ref_Declaration_Opaque_Id)
   is
   begin
      Why_Node_Id_To_Json (O, Arg);
   end W_Global_Ref_Declaration_Opaque_Id_To_Json;

   procedure W_Global_Ref_Declaration_Opaque_List_To_Json
     (O : Output_Id; Arg : W_Global_Ref_Declaration_Opaque_List);

   procedure W_Global_Ref_Declaration_Opaque_List_To_Json
     (O : Output_Id; Arg : W_Global_Ref_Declaration_Opaque_List)
   is
   begin
      Why_Node_List_To_Json (O, Arg);
   end W_Global_Ref_Declaration_Opaque_List_To_Json;

   procedure W_Namespace_Declaration_Opaque_OId_To_Json
     (O : Output_Id; Arg : W_Namespace_Declaration_Opaque_OId);

   procedure W_Namespace_Declaration_Opaque_OId_To_Json
     (O : Output_Id; Arg : W_Namespace_Declaration_Opaque_OId)
   is
   begin
      Why_Node_OId_To_Json (O, Arg);
   end W_Namespace_Declaration_Opaque_OId_To_Json;

   procedure W_Namespace_Declaration_Opaque_OList_To_Json
     (O : Output_Id; Arg : W_Namespace_Declaration_Opaque_OList);

   procedure W_Namespace_Declaration_Opaque_OList_To_Json
     (O : Output_Id; Arg : W_Namespace_Declaration_Opaque_OList)
   is
   begin
      Why_Node_OList_To_Json (O, Arg);
   end W_Namespace_Declaration_Opaque_OList_To_Json;

   procedure W_Namespace_Declaration_Opaque_Id_To_Json
     (O : Output_Id; Arg : W_Namespace_Declaration_Opaque_Id);

   procedure W_Namespace_Declaration_Opaque_Id_To_Json
     (O : Output_Id; Arg : W_Namespace_Declaration_Opaque_Id)
   is
   begin
      Why_Node_Id_To_Json (O, Arg);
   end W_Namespace_Declaration_Opaque_Id_To_Json;

   procedure W_Namespace_Declaration_Opaque_List_To_Json
     (O : Output_Id; Arg : W_Namespace_Declaration_Opaque_List);

   procedure W_Namespace_Declaration_Opaque_List_To_Json
     (O : Output_Id; Arg : W_Namespace_Declaration_Opaque_List)
   is
   begin
      Why_Node_List_To_Json (O, Arg);
   end W_Namespace_Declaration_Opaque_List_To_Json;

   procedure W_Exception_Declaration_Opaque_OId_To_Json
     (O : Output_Id; Arg : W_Exception_Declaration_Opaque_OId);

   procedure W_Exception_Declaration_Opaque_OId_To_Json
     (O : Output_Id; Arg : W_Exception_Declaration_Opaque_OId)
   is
   begin
      Why_Node_OId_To_Json (O, Arg);
   end W_Exception_Declaration_Opaque_OId_To_Json;

   procedure W_Exception_Declaration_Opaque_OList_To_Json
     (O : Output_Id; Arg : W_Exception_Declaration_Opaque_OList);

   procedure W_Exception_Declaration_Opaque_OList_To_Json
     (O : Output_Id; Arg : W_Exception_Declaration_Opaque_OList)
   is
   begin
      Why_Node_OList_To_Json (O, Arg);
   end W_Exception_Declaration_Opaque_OList_To_Json;

   procedure W_Exception_Declaration_Opaque_Id_To_Json
     (O : Output_Id; Arg : W_Exception_Declaration_Opaque_Id);

   procedure W_Exception_Declaration_Opaque_Id_To_Json
     (O : Output_Id; Arg : W_Exception_Declaration_Opaque_Id)
   is
   begin
      Why_Node_Id_To_Json (O, Arg);
   end W_Exception_Declaration_Opaque_Id_To_Json;

   procedure W_Exception_Declaration_Opaque_List_To_Json
     (O : Output_Id; Arg : W_Exception_Declaration_Opaque_List);

   procedure W_Exception_Declaration_Opaque_List_To_Json
     (O : Output_Id; Arg : W_Exception_Declaration_Opaque_List)
   is
   begin
      Why_Node_List_To_Json (O, Arg);
   end W_Exception_Declaration_Opaque_List_To_Json;

   procedure W_Meta_Declaration_Opaque_OId_To_Json
     (O : Output_Id; Arg : W_Meta_Declaration_Opaque_OId);

   procedure W_Meta_Declaration_Opaque_OId_To_Json
     (O : Output_Id; Arg : W_Meta_Declaration_Opaque_OId)
   is
   begin
      Why_Node_OId_To_Json (O, Arg);
   end W_Meta_Declaration_Opaque_OId_To_Json;

   procedure W_Meta_Declaration_Opaque_OList_To_Json
     (O : Output_Id; Arg : W_Meta_Declaration_Opaque_OList);

   procedure W_Meta_Declaration_Opaque_OList_To_Json
     (O : Output_Id; Arg : W_Meta_Declaration_Opaque_OList)
   is
   begin
      Why_Node_OList_To_Json (O, Arg);
   end W_Meta_Declaration_Opaque_OList_To_Json;

   procedure W_Meta_Declaration_Opaque_Id_To_Json
     (O : Output_Id; Arg : W_Meta_Declaration_Opaque_Id);

   procedure W_Meta_Declaration_Opaque_Id_To_Json
     (O : Output_Id; Arg : W_Meta_Declaration_Opaque_Id)
   is
   begin
      Why_Node_Id_To_Json (O, Arg);
   end W_Meta_Declaration_Opaque_Id_To_Json;

   procedure W_Meta_Declaration_Opaque_List_To_Json
     (O : Output_Id; Arg : W_Meta_Declaration_Opaque_List);

   procedure W_Meta_Declaration_Opaque_List_To_Json
     (O : Output_Id; Arg : W_Meta_Declaration_Opaque_List)
   is
   begin
      Why_Node_List_To_Json (O, Arg);
   end W_Meta_Declaration_Opaque_List_To_Json;

   procedure W_Clone_Declaration_Opaque_OId_To_Json
     (O : Output_Id; Arg : W_Clone_Declaration_Opaque_OId);

   procedure W_Clone_Declaration_Opaque_OId_To_Json
     (O : Output_Id; Arg : W_Clone_Declaration_Opaque_OId)
   is
   begin
      Why_Node_OId_To_Json (O, Arg);
   end W_Clone_Declaration_Opaque_OId_To_Json;

   procedure W_Clone_Declaration_Opaque_OList_To_Json
     (O : Output_Id; Arg : W_Clone_Declaration_Opaque_OList);

   procedure W_Clone_Declaration_Opaque_OList_To_Json
     (O : Output_Id; Arg : W_Clone_Declaration_Opaque_OList)
   is
   begin
      Why_Node_OList_To_Json (O, Arg);
   end W_Clone_Declaration_Opaque_OList_To_Json;

   procedure W_Clone_Declaration_Opaque_Id_To_Json
     (O : Output_Id; Arg : W_Clone_Declaration_Opaque_Id);

   procedure W_Clone_Declaration_Opaque_Id_To_Json
     (O : Output_Id; Arg : W_Clone_Declaration_Opaque_Id)
   is
   begin
      Why_Node_Id_To_Json (O, Arg);
   end W_Clone_Declaration_Opaque_Id_To_Json;

   procedure W_Clone_Declaration_Opaque_List_To_Json
     (O : Output_Id; Arg : W_Clone_Declaration_Opaque_List);

   procedure W_Clone_Declaration_Opaque_List_To_Json
     (O : Output_Id; Arg : W_Clone_Declaration_Opaque_List)
   is
   begin
      Why_Node_List_To_Json (O, Arg);
   end W_Clone_Declaration_Opaque_List_To_Json;

   procedure W_Clone_Substitution_Opaque_OId_To_Json
     (O : Output_Id; Arg : W_Clone_Substitution_Opaque_OId);

   procedure W_Clone_Substitution_Opaque_OId_To_Json
     (O : Output_Id; Arg : W_Clone_Substitution_Opaque_OId)
   is
   begin
      Why_Node_OId_To_Json (O, Arg);
   end W_Clone_Substitution_Opaque_OId_To_Json;

   procedure W_Clone_Substitution_Opaque_OList_To_Json
     (O : Output_Id; Arg : W_Clone_Substitution_Opaque_OList);

   procedure W_Clone_Substitution_Opaque_OList_To_Json
     (O : Output_Id; Arg : W_Clone_Substitution_Opaque_OList)
   is
   begin
      Why_Node_OList_To_Json (O, Arg);
   end W_Clone_Substitution_Opaque_OList_To_Json;

   procedure W_Clone_Substitution_Opaque_Id_To_Json
     (O : Output_Id; Arg : W_Clone_Substitution_Opaque_Id);

   procedure W_Clone_Substitution_Opaque_Id_To_Json
     (O : Output_Id; Arg : W_Clone_Substitution_Opaque_Id)
   is
   begin
      Why_Node_Id_To_Json (O, Arg);
   end W_Clone_Substitution_Opaque_Id_To_Json;

   procedure W_Clone_Substitution_Opaque_List_To_Json
     (O : Output_Id; Arg : W_Clone_Substitution_Opaque_List);

   procedure W_Clone_Substitution_Opaque_List_To_Json
     (O : Output_Id; Arg : W_Clone_Substitution_Opaque_List)
   is
   begin
      Why_Node_List_To_Json (O, Arg);
   end W_Clone_Substitution_Opaque_List_To_Json;

   procedure W_Include_Declaration_Opaque_OId_To_Json
     (O : Output_Id; Arg : W_Include_Declaration_Opaque_OId);

   procedure W_Include_Declaration_Opaque_OId_To_Json
     (O : Output_Id; Arg : W_Include_Declaration_Opaque_OId)
   is
   begin
      Why_Node_OId_To_Json (O, Arg);
   end W_Include_Declaration_Opaque_OId_To_Json;

   procedure W_Include_Declaration_Opaque_OList_To_Json
     (O : Output_Id; Arg : W_Include_Declaration_Opaque_OList);

   procedure W_Include_Declaration_Opaque_OList_To_Json
     (O : Output_Id; Arg : W_Include_Declaration_Opaque_OList)
   is
   begin
      Why_Node_OList_To_Json (O, Arg);
   end W_Include_Declaration_Opaque_OList_To_Json;

   procedure W_Include_Declaration_Opaque_Id_To_Json
     (O : Output_Id; Arg : W_Include_Declaration_Opaque_Id);

   procedure W_Include_Declaration_Opaque_Id_To_Json
     (O : Output_Id; Arg : W_Include_Declaration_Opaque_Id)
   is
   begin
      Why_Node_Id_To_Json (O, Arg);
   end W_Include_Declaration_Opaque_Id_To_Json;

   procedure W_Include_Declaration_Opaque_List_To_Json
     (O : Output_Id; Arg : W_Include_Declaration_Opaque_List);

   procedure W_Include_Declaration_Opaque_List_To_Json
     (O : Output_Id; Arg : W_Include_Declaration_Opaque_List)
   is
   begin
      Why_Node_List_To_Json (O, Arg);
   end W_Include_Declaration_Opaque_List_To_Json;

   procedure W_Theory_Declaration_Opaque_OId_To_Json
     (O : Output_Id; Arg : W_Theory_Declaration_Opaque_OId);

   procedure W_Theory_Declaration_Opaque_OId_To_Json
     (O : Output_Id; Arg : W_Theory_Declaration_Opaque_OId)
   is
   begin
      Why_Node_OId_To_Json (O, Arg);
   end W_Theory_Declaration_Opaque_OId_To_Json;

   procedure W_Theory_Declaration_Opaque_OList_To_Json
     (O : Output_Id; Arg : W_Theory_Declaration_Opaque_OList);

   procedure W_Theory_Declaration_Opaque_OList_To_Json
     (O : Output_Id; Arg : W_Theory_Declaration_Opaque_OList)
   is
   begin
      Why_Node_OList_To_Json (O, Arg);
   end W_Theory_Declaration_Opaque_OList_To_Json;

   procedure W_Theory_Declaration_Opaque_Id_To_Json
     (O : Output_Id; Arg : W_Theory_Declaration_Opaque_Id);

   procedure W_Theory_Declaration_Opaque_Id_To_Json
     (O : Output_Id; Arg : W_Theory_Declaration_Opaque_Id)
   is
   begin
      Why_Node_Id_To_Json (O, Arg);
   end W_Theory_Declaration_Opaque_Id_To_Json;

   procedure W_Theory_Declaration_Opaque_List_To_Json
     (O : Output_Id; Arg : W_Theory_Declaration_Opaque_List);

   procedure W_Theory_Declaration_Opaque_List_To_Json
     (O : Output_Id; Arg : W_Theory_Declaration_Opaque_List)
   is
   begin
      Why_Node_List_To_Json (O, Arg);
   end W_Theory_Declaration_Opaque_List_To_Json;

   procedure W_Module_Opaque_OId_To_Json
     (O : Output_Id; Arg : W_Module_Opaque_OId);

   procedure W_Module_Opaque_OId_To_Json
     (O : Output_Id; Arg : W_Module_Opaque_OId)
   is
   begin
      Why_Node_OId_To_Json (O, Arg);
   end W_Module_Opaque_OId_To_Json;

   procedure W_Module_Opaque_OList_To_Json
     (O : Output_Id; Arg : W_Module_Opaque_OList);

   procedure W_Module_Opaque_OList_To_Json
     (O : Output_Id; Arg : W_Module_Opaque_OList)
   is
   begin
      Why_Node_OList_To_Json (O, Arg);
   end W_Module_Opaque_OList_To_Json;

   procedure W_Module_Opaque_Id_To_Json
     (O : Output_Id; Arg : W_Module_Opaque_Id);

   procedure W_Module_Opaque_Id_To_Json
     (O : Output_Id; Arg : W_Module_Opaque_Id)
   is
   begin
      Why_Node_Id_To_Json (O, Arg);
   end W_Module_Opaque_Id_To_Json;

   procedure W_Module_Opaque_List_To_Json
     (O : Output_Id; Arg : W_Module_Opaque_List);

   procedure W_Module_Opaque_List_To_Json
     (O : Output_Id; Arg : W_Module_Opaque_List)
   is
   begin
      Why_Node_List_To_Json (O, Arg);
   end W_Module_Opaque_List_To_Json;

   procedure W_Expr_Opaque_OId_To_Json
     (O : Output_Id; Arg : W_Expr_Opaque_OId);

   procedure W_Expr_Opaque_OId_To_Json
     (O : Output_Id; Arg : W_Expr_Opaque_OId)
   is
   begin
      Why_Node_OId_To_Json (O, Arg);
   end W_Expr_Opaque_OId_To_Json;

   procedure W_Expr_Opaque_OList_To_Json
     (O : Output_Id; Arg : W_Expr_Opaque_OList);

   procedure W_Expr_Opaque_OList_To_Json
     (O : Output_Id; Arg : W_Expr_Opaque_OList)
   is
   begin
      Why_Node_OList_To_Json (O, Arg);
   end W_Expr_Opaque_OList_To_Json;

   procedure W_Expr_Opaque_Id_To_Json
     (O : Output_Id; Arg : W_Expr_Opaque_Id);

   procedure W_Expr_Opaque_Id_To_Json
     (O : Output_Id; Arg : W_Expr_Opaque_Id)
   is
   begin
      Why_Node_Id_To_Json (O, Arg);
   end W_Expr_Opaque_Id_To_Json;

   procedure W_Expr_Opaque_List_To_Json
     (O : Output_Id; Arg : W_Expr_Opaque_List);

   procedure W_Expr_Opaque_List_To_Json
     (O : Output_Id; Arg : W_Expr_Opaque_List)
   is
   begin
      Why_Node_List_To_Json (O, Arg);
   end W_Expr_Opaque_List_To_Json;

   procedure W_Pred_Opaque_OId_To_Json
     (O : Output_Id; Arg : W_Pred_Opaque_OId);

   procedure W_Pred_Opaque_OId_To_Json
     (O : Output_Id; Arg : W_Pred_Opaque_OId)
   is
   begin
      Why_Node_OId_To_Json (O, Arg);
   end W_Pred_Opaque_OId_To_Json;

   procedure W_Pred_Opaque_OList_To_Json
     (O : Output_Id; Arg : W_Pred_Opaque_OList);

   procedure W_Pred_Opaque_OList_To_Json
     (O : Output_Id; Arg : W_Pred_Opaque_OList)
   is
   begin
      Why_Node_OList_To_Json (O, Arg);
   end W_Pred_Opaque_OList_To_Json;

   procedure W_Pred_Opaque_Id_To_Json
     (O : Output_Id; Arg : W_Pred_Opaque_Id);

   procedure W_Pred_Opaque_Id_To_Json
     (O : Output_Id; Arg : W_Pred_Opaque_Id)
   is
   begin
      Why_Node_Id_To_Json (O, Arg);
   end W_Pred_Opaque_Id_To_Json;

   procedure W_Pred_Opaque_List_To_Json
     (O : Output_Id; Arg : W_Pred_Opaque_List);

   procedure W_Pred_Opaque_List_To_Json
     (O : Output_Id; Arg : W_Pred_Opaque_List)
   is
   begin
      Why_Node_List_To_Json (O, Arg);
   end W_Pred_Opaque_List_To_Json;

   procedure W_Term_Opaque_OId_To_Json
     (O : Output_Id; Arg : W_Term_Opaque_OId);

   procedure W_Term_Opaque_OId_To_Json
     (O : Output_Id; Arg : W_Term_Opaque_OId)
   is
   begin
      Why_Node_OId_To_Json (O, Arg);
   end W_Term_Opaque_OId_To_Json;

   procedure W_Term_Opaque_OList_To_Json
     (O : Output_Id; Arg : W_Term_Opaque_OList);

   procedure W_Term_Opaque_OList_To_Json
     (O : Output_Id; Arg : W_Term_Opaque_OList)
   is
   begin
      Why_Node_OList_To_Json (O, Arg);
   end W_Term_Opaque_OList_To_Json;

   procedure W_Term_Opaque_Id_To_Json
     (O : Output_Id; Arg : W_Term_Opaque_Id);

   procedure W_Term_Opaque_Id_To_Json
     (O : Output_Id; Arg : W_Term_Opaque_Id)
   is
   begin
      Why_Node_Id_To_Json (O, Arg);
   end W_Term_Opaque_Id_To_Json;

   procedure W_Term_Opaque_List_To_Json
     (O : Output_Id; Arg : W_Term_Opaque_List);

   procedure W_Term_Opaque_List_To_Json
     (O : Output_Id; Arg : W_Term_Opaque_List)
   is
   begin
      Why_Node_List_To_Json (O, Arg);
   end W_Term_Opaque_List_To_Json;

   procedure W_Prog_Opaque_OId_To_Json
     (O : Output_Id; Arg : W_Prog_Opaque_OId);

   procedure W_Prog_Opaque_OId_To_Json
     (O : Output_Id; Arg : W_Prog_Opaque_OId)
   is
   begin
      Why_Node_OId_To_Json (O, Arg);
   end W_Prog_Opaque_OId_To_Json;

   procedure W_Prog_Opaque_OList_To_Json
     (O : Output_Id; Arg : W_Prog_Opaque_OList);

   procedure W_Prog_Opaque_OList_To_Json
     (O : Output_Id; Arg : W_Prog_Opaque_OList)
   is
   begin
      Why_Node_OList_To_Json (O, Arg);
   end W_Prog_Opaque_OList_To_Json;

   procedure W_Prog_Opaque_Id_To_Json
     (O : Output_Id; Arg : W_Prog_Opaque_Id);

   procedure W_Prog_Opaque_Id_To_Json
     (O : Output_Id; Arg : W_Prog_Opaque_Id)
   is
   begin
      Why_Node_Id_To_Json (O, Arg);
   end W_Prog_Opaque_Id_To_Json;

   procedure W_Prog_Opaque_List_To_Json
     (O : Output_Id; Arg : W_Prog_Opaque_List);

   procedure W_Prog_Opaque_List_To_Json
     (O : Output_Id; Arg : W_Prog_Opaque_List)
   is
   begin
      Why_Node_List_To_Json (O, Arg);
   end W_Prog_Opaque_List_To_Json;

   procedure W_Type_Definition_Opaque_OId_To_Json
     (O : Output_Id; Arg : W_Type_Definition_Opaque_OId);

   procedure W_Type_Definition_Opaque_OId_To_Json
     (O : Output_Id; Arg : W_Type_Definition_Opaque_OId)
   is
   begin
      Why_Node_OId_To_Json (O, Arg);
   end W_Type_Definition_Opaque_OId_To_Json;

   procedure W_Type_Definition_Opaque_OList_To_Json
     (O : Output_Id; Arg : W_Type_Definition_Opaque_OList);

   procedure W_Type_Definition_Opaque_OList_To_Json
     (O : Output_Id; Arg : W_Type_Definition_Opaque_OList)
   is
   begin
      Why_Node_OList_To_Json (O, Arg);
   end W_Type_Definition_Opaque_OList_To_Json;

   procedure W_Type_Definition_Opaque_Id_To_Json
     (O : Output_Id; Arg : W_Type_Definition_Opaque_Id);

   procedure W_Type_Definition_Opaque_Id_To_Json
     (O : Output_Id; Arg : W_Type_Definition_Opaque_Id)
   is
   begin
      Why_Node_Id_To_Json (O, Arg);
   end W_Type_Definition_Opaque_Id_To_Json;

   procedure W_Type_Definition_Opaque_List_To_Json
     (O : Output_Id; Arg : W_Type_Definition_Opaque_List);

   procedure W_Type_Definition_Opaque_List_To_Json
     (O : Output_Id; Arg : W_Type_Definition_Opaque_List)
   is
   begin
      Why_Node_List_To_Json (O, Arg);
   end W_Type_Definition_Opaque_List_To_Json;

   procedure W_Declaration_Opaque_OId_To_Json
     (O : Output_Id; Arg : W_Declaration_Opaque_OId);

   procedure W_Declaration_Opaque_OId_To_Json
     (O : Output_Id; Arg : W_Declaration_Opaque_OId)
   is
   begin
      Why_Node_OId_To_Json (O, Arg);
   end W_Declaration_Opaque_OId_To_Json;

   procedure W_Declaration_Opaque_OList_To_Json
     (O : Output_Id; Arg : W_Declaration_Opaque_OList);

   procedure W_Declaration_Opaque_OList_To_Json
     (O : Output_Id; Arg : W_Declaration_Opaque_OList)
   is
   begin
      Why_Node_OList_To_Json (O, Arg);
   end W_Declaration_Opaque_OList_To_Json;

   procedure W_Declaration_Opaque_Id_To_Json
     (O : Output_Id; Arg : W_Declaration_Opaque_Id);

   procedure W_Declaration_Opaque_Id_To_Json
     (O : Output_Id; Arg : W_Declaration_Opaque_Id)
   is
   begin
      Why_Node_Id_To_Json (O, Arg);
   end W_Declaration_Opaque_Id_To_Json;

   procedure W_Declaration_Opaque_List_To_Json
     (O : Output_Id; Arg : W_Declaration_Opaque_List);

   procedure W_Declaration_Opaque_List_To_Json
     (O : Output_Id; Arg : W_Declaration_Opaque_List)
   is
   begin
      Why_Node_List_To_Json (O, Arg);
   end W_Declaration_Opaque_List_To_Json;

   procedure W_Any_Node_Opaque_OId_To_Json
     (O : Output_Id; Arg : W_Any_Node_Opaque_OId);

   procedure W_Any_Node_Opaque_OId_To_Json
     (O : Output_Id; Arg : W_Any_Node_Opaque_OId)
   is
   begin
      Why_Node_OId_To_Json (O, Arg);
   end W_Any_Node_Opaque_OId_To_Json;

   procedure W_Any_Node_Opaque_OList_To_Json
     (O : Output_Id; Arg : W_Any_Node_Opaque_OList);

   procedure W_Any_Node_Opaque_OList_To_Json
     (O : Output_Id; Arg : W_Any_Node_Opaque_OList)
   is
   begin
      Why_Node_OList_To_Json (O, Arg);
   end W_Any_Node_Opaque_OList_To_Json;

   procedure W_Any_Node_Opaque_Id_To_Json
     (O : Output_Id; Arg : W_Any_Node_Opaque_Id);

   procedure W_Any_Node_Opaque_Id_To_Json
     (O : Output_Id; Arg : W_Any_Node_Opaque_Id)
   is
   begin
      Why_Node_Id_To_Json (O, Arg);
   end W_Any_Node_Opaque_Id_To_Json;

   procedure W_Any_Node_Opaque_List_To_Json
     (O : Output_Id; Arg : W_Any_Node_Opaque_List);

   procedure W_Any_Node_Opaque_List_To_Json
     (O : Output_Id; Arg : W_Any_Node_Opaque_List)
   is
   begin
      Why_Node_List_To_Json (O, Arg);
   end W_Any_Node_Opaque_List_To_Json;

   Why_Node_Counter : Natural := 0;

   procedure Why_Node_To_Json (O : Output_Id; Node : Why_Node) is
   begin
      P (O, '[');
      Why_Node_Counter := Why_Node_Counter + 1;
      P (O, '"' & Why_Node_Kind'Image (Node.Kind) & '"');
      P (O, ',');
      P (O, Integer'Image (Why_Node_Counter));
      P (O, ',');
      EW_Domain_To_Json (O, Node.Domain);
      case Node.Kind is
         when W_Unused_At_Start =>
            null;
         when W_Type =>
            P (O, ',');
            EW_Type_To_Json
              (O,
               Node.K1_Type_Kind);
            P (O, ',');
            W_Name_Opaque_Id_To_Json
              (O,
               Node.K1_Name);
            P (O, ',');
            Boolean_To_Json
              (O,
               Node.K1_Is_Mutable);
            P (O, ',');
            Boolean_To_Json
              (O,
               Node.K1_Relaxed_Init);
         when W_Name =>
            P (O, ',');
            Symbol_To_Json
              (O,
               Node.K2_Symb);
            P (O, ',');
            Symbol_To_Json
              (O,
               Node.K2_Namespace);
            P (O, ',');
            W_Module_Opaque_OId_To_Json
              (O,
               Node.K2_Module);
            P (O, ',');
            Boolean_To_Json
              (O,
               Node.K2_Infix);
         when W_Effects =>
            P (O, ',');
            W_Identifier_Opaque_OList_To_Json
              (O,
               Node.K3_Reads);
            P (O, ',');
            W_Identifier_Opaque_OList_To_Json
              (O,
               Node.K3_Writes);
            P (O, ',');
            W_Raise_Effect_Opaque_OList_To_Json
              (O,
               Node.K3_Raises);
         when W_Raise_Effect =>
            P (O, ',');
            W_Name_Opaque_Id_To_Json
              (O,
               Node.K4_Name);
            P (O, ',');
            W_Identifier_Opaque_OId_To_Json
              (O,
               Node.K4_Arg_Id);
            P (O, ',');
            W_Pred_Opaque_OId_To_Json
              (O,
               Node.K4_Post);
         when W_Binder =>
            P (O, ',');
            W_Identifier_Opaque_OId_To_Json
              (O,
               Node.K5_Name);
            P (O, ',');
            W_Type_Opaque_Id_To_Json
              (O,
               Node.K5_Arg_Type);
         when W_Transparent_Type_Definition =>
            P (O, ',');
            W_Type_Opaque_Id_To_Json
              (O,
               Node.K6_Type_Definition);
         when W_Record_Binder =>
            P (O, ',');
            W_Identifier_Opaque_OId_To_Json
              (O,
               Node.K7_Name);
            P (O, ',');
            W_Type_Opaque_Id_To_Json
              (O,
               Node.K7_Arg_Type);
            P (O, ',');
            Symbol_Set_To_Json
              (O,
               Node.K7_Labels);
            P (O, ',');
            Boolean_To_Json
              (O,
               Node.K7_Is_Mutable);
         when W_Record_Definition =>
            P (O, ',');
            W_Record_Binder_Opaque_List_To_Json
              (O,
               Node.K8_Fields);
         when W_Range_Type_Definition =>
            P (O, ',');
            Uint_To_Json
              (O,
               Node.K9_First);
            P (O, ',');
            Uint_To_Json
              (O,
               Node.K9_Last);
         when W_Triggers =>
            P (O, ',');
            W_Trigger_Opaque_List_To_Json
              (O,
               Node.K10_Triggers);
         when W_Trigger =>
            P (O, ',');
            W_Expr_Opaque_List_To_Json
              (O,
               Node.K11_Terms);
         when W_Axiom_Dep =>
            P (O, ',');
            W_Identifier_Opaque_Id_To_Json
              (O,
               Node.K12_Name);
            P (O, ',');
            EW_Axiom_Dep_Kind_To_Json
              (O,
               Node.K12_Kind);
         when W_Handler =>
            P (O, ',');
            W_Name_Opaque_Id_To_Json
              (O,
               Node.K13_Name);
            P (O, ',');
            W_Identifier_Opaque_OId_To_Json
              (O,
               Node.K13_Arg_Id);
            P (O, ',');
            W_Prog_Opaque_Id_To_Json
              (O,
               Node.K13_Def);
         when W_Field_Association =>
            P (O, ',');
            W_Identifier_Opaque_Id_To_Json
              (O,
               Node.K14_Field);
            P (O, ',');
            W_Expr_Opaque_Id_To_Json
              (O,
               Node.K14_Value);
         when W_Variant =>
            P (O, ',');
            W_Identifier_Opaque_Id_To_Json
              (O,
               Node.K15_Cmp_Op);
            P (O, ',');
            Symbol_Set_To_Json
              (O,
               Node.K15_Labels);
            P (O, ',');
            W_Term_Opaque_Id_To_Json
              (O,
               Node.K15_Expr);
         when W_Variants =>
            P (O, ',');
            W_Variant_Opaque_List_To_Json
              (O,
               Node.K16_Variants);
         when W_Universal_Quantif =>
            P (O, ',');
            W_Identifier_Opaque_List_To_Json
              (O,
               Node.K17_Variables);
            P (O, ',');
            Symbol_Set_To_Json
              (O,
               Node.K17_Labels);
            P (O, ',');
            W_Type_Opaque_Id_To_Json
              (O,
               Node.K17_Var_Type);
            P (O, ',');
            W_Triggers_Opaque_OId_To_Json
              (O,
               Node.K17_Triggers);
            P (O, ',');
            W_Pred_Opaque_Id_To_Json
              (O,
               Node.K17_Pred);
         when W_Existential_Quantif =>
            P (O, ',');
            W_Identifier_Opaque_List_To_Json
              (O,
               Node.K18_Variables);
            P (O, ',');
            Symbol_Set_To_Json
              (O,
               Node.K18_Labels);
            P (O, ',');
            W_Type_Opaque_Id_To_Json
              (O,
               Node.K18_Var_Type);
            P (O, ',');
            W_Pred_Opaque_Id_To_Json
              (O,
               Node.K18_Pred);
         when W_Not =>
            P (O, ',');
            W_Expr_Opaque_Id_To_Json
              (O,
               Node.K19_Right);
         when W_Connection =>
            P (O, ',');
            W_Expr_Opaque_Id_To_Json
              (O,
               Node.K20_Left);
            P (O, ',');
            EW_Connector_To_Json
              (O,
               Node.K20_Op);
            P (O, ',');
            W_Expr_Opaque_Id_To_Json
              (O,
               Node.K20_Right);
            P (O, ',');
            W_Expr_Opaque_OList_To_Json
              (O,
               Node.K20_More_Right);
         when W_Label =>
            P (O, ',');
            Symbol_Set_To_Json
              (O,
               Node.K21_Labels);
            P (O, ',');
            W_Expr_Opaque_Id_To_Json
              (O,
               Node.K21_Def);
            P (O, ',');
            W_Type_Opaque_OId_To_Json
              (O,
               Node.K21_Typ);
         when W_Loc_Label =>
            P (O, ',');
            Source_Ptr_To_Json
              (O,
               Node.K22_Sloc);
            P (O, ',');
            W_Expr_Opaque_Id_To_Json
              (O,
               Node.K22_Def);
            P (O, ',');
            Symbol_To_Json
              (O,
               Node.K22_Marker);
         when W_Identifier =>
            P (O, ',');
            W_Name_Opaque_Id_To_Json
              (O,
               Node.K23_Name);
            P (O, ',');
            W_Type_Opaque_OId_To_Json
              (O,
               Node.K23_Typ);
            P (O, ',');
            String_Sets_Set_To_Json
              (O,
               Node.K23_Labels);
         when W_Tagged =>
            P (O, ',');
            Symbol_To_Json
              (O,
               Node.K24_Tag);
            P (O, ',');
            W_Expr_Opaque_Id_To_Json
              (O,
               Node.K24_Def);
            P (O, ',');
            W_Type_Opaque_OId_To_Json
              (O,
               Node.K24_Typ);
         when W_Call =>
            P (O, ',');
            W_Identifier_Opaque_Id_To_Json
              (O,
               Node.K25_Name);
            P (O, ',');
            W_Expr_Opaque_OList_To_Json
              (O,
               Node.K25_Args);
            P (O, ',');
            W_Type_Opaque_OId_To_Json
              (O,
               Node.K25_Typ);
         when W_Literal =>
            P (O, ',');
            EW_Literal_To_Json
              (O,
               Node.K26_Value);
            P (O, ',');
            W_Type_Opaque_OId_To_Json
              (O,
               Node.K26_Typ);
         when W_Binding =>
            P (O, ',');
            W_Identifier_Opaque_Id_To_Json
              (O,
               Node.K27_Name);
            P (O, ',');
            W_Expr_Opaque_Id_To_Json
              (O,
               Node.K27_Def);
            P (O, ',');
            W_Expr_Opaque_Id_To_Json
              (O,
               Node.K27_Context);
            P (O, ',');
            W_Type_Opaque_OId_To_Json
              (O,
               Node.K27_Typ);
         when W_Elsif =>
            P (O, ',');
            W_Expr_Opaque_Id_To_Json
              (O,
               Node.K28_Condition);
            P (O, ',');
            W_Expr_Opaque_Id_To_Json
              (O,
               Node.K28_Then_Part);
            P (O, ',');
            W_Type_Opaque_OId_To_Json
              (O,
               Node.K28_Typ);
         when W_Epsilon =>
            P (O, ',');
            W_Identifier_Opaque_Id_To_Json
              (O,
               Node.K29_Name);
            P (O, ',');
            W_Type_Opaque_Id_To_Json
              (O,
               Node.K29_Typ);
            P (O, ',');
            W_Pred_Opaque_Id_To_Json
              (O,
               Node.K29_Pred);
         when W_Conditional =>
            P (O, ',');
            W_Expr_Opaque_Id_To_Json
              (O,
               Node.K30_Condition);
            P (O, ',');
            W_Expr_Opaque_Id_To_Json
              (O,
               Node.K30_Then_Part);
            P (O, ',');
            W_Expr_Opaque_OList_To_Json
              (O,
               Node.K30_Elsif_Parts);
            P (O, ',');
            W_Expr_Opaque_OId_To_Json
              (O,
               Node.K30_Else_Part);
            P (O, ',');
            W_Type_Opaque_OId_To_Json
              (O,
               Node.K30_Typ);
         when W_Integer_Constant =>
            P (O, ',');
            Uint_To_Json
              (O,
               Node.K31_Value);
         when W_Range_Constant =>
            P (O, ',');
            Uint_To_Json
              (O,
               Node.K32_Value);
            P (O, ',');
            W_Type_Opaque_Id_To_Json
              (O,
               Node.K32_Typ);
         when W_Modular_Constant =>
            P (O, ',');
            Uint_To_Json
              (O,
               Node.K33_Value);
            P (O, ',');
            W_Type_Opaque_Id_To_Json
              (O,
               Node.K33_Typ);
         when W_Fixed_Constant =>
            P (O, ',');
            Uint_To_Json
              (O,
               Node.K34_Value);
            P (O, ',');
            W_Type_Opaque_Id_To_Json
              (O,
               Node.K34_Typ);
         when W_Real_Constant =>
            P (O, ',');
            Ureal_To_Json
              (O,
               Node.K35_Value);
         when W_Float_Constant =>
            P (O, ',');
            Ureal_To_Json
              (O,
               Node.K36_Value);
            P (O, ',');
            W_Type_Opaque_Id_To_Json
              (O,
               Node.K36_Typ);
         when W_Comment =>
            P (O, ',');
            Symbol_To_Json
              (O,
               Node.K37_Comment);
         when W_Deref =>
            P (O, ',');
            W_Identifier_Opaque_Id_To_Json
              (O,
               Node.K38_Right);
            P (O, ',');
            W_Type_Opaque_Id_To_Json
              (O,
               Node.K38_Typ);
         when W_Record_Access =>
            P (O, ',');
            W_Expr_Opaque_Id_To_Json
              (O,
               Node.K39_Name);
            P (O, ',');
            W_Identifier_Opaque_Id_To_Json
              (O,
               Node.K39_Field);
            P (O, ',');
            W_Type_Opaque_OId_To_Json
              (O,
               Node.K39_Typ);
         when W_Record_Update =>
            P (O, ',');
            W_Expr_Opaque_Id_To_Json
              (O,
               Node.K40_Name);
            P (O, ',');
            W_Field_Association_Opaque_List_To_Json
              (O,
               Node.K40_Updates);
            P (O, ',');
            W_Type_Opaque_OId_To_Json
              (O,
               Node.K40_Typ);
         when W_Record_Aggregate =>
            P (O, ',');
            W_Field_Association_Opaque_List_To_Json
              (O,
               Node.K41_Associations);
            P (O, ',');
            W_Type_Opaque_OId_To_Json
              (O,
               Node.K41_Typ);
         when W_Any_Expr =>
            P (O, ',');
            W_Effects_Opaque_OId_To_Json
              (O,
               Node.K42_Effects);
            P (O, ',');
            W_Pred_Opaque_OId_To_Json
              (O,
               Node.K42_Pre);
            P (O, ',');
            W_Pred_Opaque_OId_To_Json
              (O,
               Node.K42_Post);
            P (O, ',');
            W_Type_Opaque_Id_To_Json
              (O,
               Node.K42_Return_Type);
            P (O, ',');
            Symbol_Set_To_Json
              (O,
               Node.K42_Labels);
         when W_Assignment =>
            P (O, ',');
            W_Identifier_Opaque_Id_To_Json
              (O,
               Node.K43_Name);
            P (O, ',');
            W_Prog_Opaque_Id_To_Json
              (O,
               Node.K43_Value);
            P (O, ',');
            W_Type_Opaque_Id_To_Json
              (O,
               Node.K43_Typ);
            P (O, ',');
            Symbol_Set_To_Json
              (O,
               Node.K43_Labels);
         when W_Binding_Ref =>
            P (O, ',');
            W_Identifier_Opaque_Id_To_Json
              (O,
               Node.K44_Name);
            P (O, ',');
            W_Prog_Opaque_Id_To_Json
              (O,
               Node.K44_Def);
            P (O, ',');
            W_Prog_Opaque_Id_To_Json
              (O,
               Node.K44_Context);
            P (O, ',');
            W_Type_Opaque_Id_To_Json
              (O,
               Node.K44_Typ);
         when W_Loop =>
            P (O, ',');
            W_Prog_Opaque_Id_To_Json
              (O,
               Node.K45_Code_Before);
            P (O, ',');
            W_Pred_Opaque_OList_To_Json
              (O,
               Node.K45_Invariants);
            P (O, ',');
            W_Variants_Opaque_OList_To_Json
              (O,
               Node.K45_Variants);
            P (O, ',');
            W_Prog_Opaque_Id_To_Json
              (O,
               Node.K45_Code_After);
         when W_Statement_Sequence =>
            P (O, ',');
            W_Prog_Opaque_List_To_Json
              (O,
               Node.K46_Statements);
         when W_Abstract_Expr =>
            P (O, ',');
            W_Prog_Opaque_Id_To_Json
              (O,
               Node.K47_Expr);
            P (O, ',');
            W_Pred_Opaque_Id_To_Json
              (O,
               Node.K47_Post);
            P (O, ',');
            W_Type_Opaque_OId_To_Json
              (O,
               Node.K47_Typ);
         when W_Assert =>
            P (O, ',');
            W_Pred_Opaque_Id_To_Json
              (O,
               Node.K48_Pred);
            P (O, ',');
            EW_Assert_Kind_To_Json
              (O,
               Node.K48_Assert_Kind);
         when W_Raise =>
            P (O, ',');
            W_Name_Opaque_Id_To_Json
              (O,
               Node.K49_Name);
            P (O, ',');
            W_Expr_Opaque_OId_To_Json
              (O,
               Node.K49_Arg);
            P (O, ',');
            W_Type_Opaque_OId_To_Json
              (O,
               Node.K49_Typ);
         when W_Try_Block =>
            P (O, ',');
            W_Prog_Opaque_Id_To_Json
              (O,
               Node.K50_Prog);
            P (O, ',');
            W_Handler_Opaque_List_To_Json
              (O,
               Node.K50_Handler);
            P (O, ',');
            W_Type_Opaque_OId_To_Json
              (O,
               Node.K50_Typ);
         when W_Function_Decl =>
            P (O, ',');
            W_Identifier_Opaque_Id_To_Json
              (O,
               Node.K51_Name);
            P (O, ',');
            W_Binder_Opaque_OList_To_Json
              (O,
               Node.K51_Binders);
            P (O, ',');
            W_Effects_Opaque_OId_To_Json
              (O,
               Node.K51_Effects);
            P (O, ',');
            W_Pred_Opaque_OId_To_Json
              (O,
               Node.K51_Pre);
            P (O, ',');
            W_Pred_Opaque_OId_To_Json
              (O,
               Node.K51_Post);
            P (O, ',');
            W_Type_Opaque_OId_To_Json
              (O,
               Node.K51_Return_Type);
            P (O, ',');
            W_Expr_Opaque_OId_To_Json
              (O,
               Node.K51_Def);
            P (O, ',');
            Symbol_Set_To_Json
              (O,
               Node.K51_Labels);
            P (O, ',');
            Source_Ptr_To_Json
              (O,
               Node.K51_Location);
         when W_Axiom =>
            P (O, ',');
            Symbol_To_Json
              (O,
               Node.K52_Name);
            P (O, ',');
            W_Pred_Opaque_Id_To_Json
              (O,
               Node.K52_Def);
            P (O, ',');
            W_Axiom_Dep_Opaque_OId_To_Json
              (O,
               Node.K52_Dep);
         when W_Goal =>
            P (O, ',');
            Symbol_To_Json
              (O,
               Node.K53_Name);
            P (O, ',');
            W_Pred_Opaque_Id_To_Json
              (O,
               Node.K53_Def);
         when W_Type_Decl =>
            P (O, ',');
            W_Identifier_Opaque_OList_To_Json
              (O,
               Node.K54_Args);
            P (O, ',');
            W_Name_Opaque_Id_To_Json
              (O,
               Node.K54_Name);
            P (O, ',');
            Symbol_Set_To_Json
              (O,
               Node.K54_Labels);
            P (O, ',');
            W_Type_Definition_Opaque_OId_To_Json
              (O,
               Node.K54_Definition);
         when W_Global_Ref_Declaration =>
            P (O, ',');
            W_Identifier_Opaque_Id_To_Json
              (O,
               Node.K55_Name);
            P (O, ',');
            W_Type_Opaque_Id_To_Json
              (O,
               Node.K55_Ref_Type);
            P (O, ',');
            Symbol_Set_To_Json
              (O,
               Node.K55_Labels);
            P (O, ',');
            Source_Ptr_To_Json
              (O,
               Node.K55_Location);
         when W_Namespace_Declaration =>
            P (O, ',');
            W_Declaration_Opaque_OList_To_Json
              (O,
               Node.K56_Declarations);
            P (O, ',');
            Symbol_To_Json
              (O,
               Node.K56_Name);
         when W_Exception_Declaration =>
            P (O, ',');
            W_Name_Opaque_Id_To_Json
              (O,
               Node.K57_Name);
            P (O, ',');
            W_Type_Opaque_OId_To_Json
              (O,
               Node.K57_Arg);
         when W_Meta_Declaration =>
            P (O, ',');
            Symbol_To_Json
              (O,
               Node.K58_Name);
            P (O, ',');
            Symbol_To_Json
              (O,
               Node.K58_Parameter);
         when W_Clone_Declaration =>
            P (O, ',');
            W_Module_Opaque_Id_To_Json
              (O,
               Node.K59_Origin);
            P (O, ',');
            Symbol_To_Json
              (O,
               Node.K59_As_Name);
            P (O, ',');
            EW_Clone_Type_To_Json
              (O,
               Node.K59_Clone_Kind);
            P (O, ',');
            W_Clone_Substitution_Opaque_OList_To_Json
              (O,
               Node.K59_Substitutions);
            P (O, ',');
            EW_Theory_Type_To_Json
              (O,
               Node.K59_Theory_Kind);
         when W_Clone_Substitution =>
            P (O, ',');
            EW_Subst_Type_To_Json
              (O,
               Node.K60_Kind);
            P (O, ',');
            W_Name_Opaque_Id_To_Json
              (O,
               Node.K60_Orig_Name);
            P (O, ',');
            W_Name_Opaque_Id_To_Json
              (O,
               Node.K60_Image);
         when W_Include_Declaration =>
            P (O, ',');
            W_Module_Opaque_Id_To_Json
              (O,
               Node.K61_Module);
            P (O, ',');
            EW_Theory_Type_To_Json
              (O,
               Node.K61_Kind);
            P (O, ',');
            EW_Clone_Type_To_Json
              (O,
               Node.K61_Use_Kind);
         when W_Theory_Declaration =>
            P (O, ',');
            W_Declaration_Opaque_OList_To_Json
              (O,
               Node.K62_Declarations);
            P (O, ',');
            Symbol_To_Json
              (O,
               Node.K62_Name);
            P (O, ',');
            EW_Theory_Type_To_Json
              (O,
               Node.K62_Kind);
            P (O, ',');
            W_Include_Declaration_Opaque_OList_To_Json
              (O,
               Node.K62_Includes);
            P (O, ',');
            Symbol_To_Json
              (O,
               Node.K62_Comment);
         when W_Module =>
            P (O, ',');
            Symbol_To_Json
              (O,
               Node.K63_File);
            P (O, ',');
            Symbol_To_Json
              (O,
               Node.K63_Name);
      end case;
      P (O, ']');
   end Why_Node_To_Json;
   pragma Warnings (On, "procedure * is not referenced");
end Why.Atree.To_Json;